/*
	To upload through terminal you can use: curl -F "image=@firmware.bin" esp32-webupdate.local/update
*/

#include <WiFi.h>
#include <WiFiClient.h>
#include <WebServer.h>
#include <ESPmDNS.h>
#include <Wire.h>
#include <EEPROM.h>
#include "driver/spi_slave.h"
#include <SPI.h>

//#include <WiFiUdp.h>
//#include <ArduinoOTA.h>

const char* host = "esp32-webupdate";

/*
	history of colorcompass WiFi model
						by AT-SYSTEM
	"0.1"....ssii,pawd size miss
	"1.1"....ssii,pawd size miss correct
	"1.2"....2022/11/23
				append config confirmation
				fix subnet mask 255.255.255.0
	"1.3"....2022/12/23
				fix AP mode IP Address 192.168.0.100
				ssid,password..16 bytes -> 32 bytes
				hard2_buf content new ssid, password, ip address
	"1.4"....2023/04/22
				add dasoku to ip at config.js
				check ip first is not 0
	note:
			128kB eeprom
			0x00000	parameter buf
			0x04000	hard buf ( sensor, dark )
			0x06000	hard2 buf( white )
			0x08000	referencr( 6 data )
			0x0b000 empty space ( 84kb )
			0x1ffff end
*/
const	char	firm_ver[ 16] = { "1.4"};

#define	DASOKU_LEN	12
const	char	dasoku[DASOKU_LEN+1]={ "colorcompass"};
const	IPAddress subnetmask(255, 255, 255, 0);
const	IPAddress ap_ip(192,168,0,100);

WebServer		server(80);


#define EEP_SCL		19
#define EEP_SDA		18
#define EEP_ADRS0	0x50
#define EEP_ADRS1	0x51

#define IO23		23

#define AN6			34

#define	MAX_PIXEL	288

#define	START_WAVE	340
#define	END_WAVE	850
#define	WAVE_NUM	( END_WAVE - START_WAVE + 1)
#define	MAX_WAVE	((WAVE_NUM+1) / 2)

typedef	struct	{
	int		exp;
	int		cyc;
	int		avg;
	int		index;
	float	temperature;
	unsigned short	pixel[ MAX_PIXEL];
} ESP_BUF;

ESP_BUF	esp32_out;
ESP_BUF	esp32_in;


char	html_buf[ 	4096];
char	style_buf[ 	4096];
char	js_buf[ 	32768];
char	json_buf[ 	16384];

char	config_html_buf[ 	4096];
char	config_js_buf[ 		4096];



static const uint32_t TRANS_SIZE = sizeof( ESP_BUF);

typedef	struct	{
	int		para_save;
	int		para_adr;
	int		kick_data;
	int		kick_dark;
	int		kick_white;
	int		kick_regist;
	int		kick_select;
	int 	eeprom_ok;
	int 	wifi_ok;
	float	temp_now;
	int		data_save;
	int		option_sw;
	int		graph_repeat;
	int		graph_mode;
	int		graph_disp;
	tm		time_now;
	float	peak;
	int		pwave;
	float	ndiv;
	int		mode;
	int		ref_raw_no;
	int		ref_reflect_no;
	int		data_kind;
	int		hard_save;
	int		hard_adr;
	int		hard2_save;
	int		hard2_adr;
} state_buf;

enum	{
	mode_raw = 0,
	mode_reflect,
};

enum	{
	disp_data = 0,
	disp_comp,
	disp_ref
};

enum	{
	mode_STA = 0,
	mode_AP,
};


state_buf	state;

typedef struct {
	int				size;						// this size
	char			firm[ 16];					// firm version 
	char			cc_name[ 32];				// colorcompass name
	char			cc_sensor[ 16];				// sensor name
	char			cc_serial[ 16];				// sensor serial
	double			cc_comp[ 6];				// sensor parameter
	unsigned short	pixel_offset[ MAX_PIXEL];	// offset
	float			pixel_gain[ MAX_PIXEL];		// pixel gain
	int				dummy;						// dummy for 8-byte boundary check sum
	unsigned int	sum;						// check sum 
} hard_buf;

typedef	struct	{
	int				size;						// this size
	char			wifi_ssid[ 32];				// wifi ssid
	char			wifi_pawd[ 32];				// wifi password
	unsigned char	ip[ 4];						// ip address
	int				dummy;						// dummy for 8-byte boundary check sum
	unsigned int	sum;						// check sum 
} hard2_buf;

typedef struct {
	int				size;						// this size
	char			wifi_ssid[ 16];				// wifi ssid
	char			wifi_pawd[ 16];				// wifi password
	unsigned char	ip[ 4];						// ip address
	int				cc_exposure;				// colorcompass exposure time (us)
	int				cc_average;					// average num
	unsigned short	pixel_white[ MAX_PIXEL];	// white
	int				wave1;
	int				wave2;
	int				dummy;						// dummy for 8-byte boundary check sum
	unsigned int	sum;						// check sum 
} para_buf;

#define	PARA_ADR	0
#define	HARD_ADR	0x4000
#define	HARD2_ADR	0x6000
#define	REF_ADR		0x8000
#define	REF_STEP	2048

#define	EP_RPAGE	64
#define	EP_WPAGE	64

hard_buf	hard;
hard2_buf	hard2;

hard_buf	hard_chg;
hard2_buf	hard2_chg;

para_buf	para;
para_buf	para_chg;								// save parameter only changed



float		pixel_wave[ MAX_PIXEL];				// 340.5, 345,5.....
float		pixel_data[ MAX_PIXEL];				// pixel raw data - dark & average
unsigned short	  dark[ MAX_PIXEL];

typedef	struct	{
	float	data[ MAX_WAVE];
	int		exp;
	float	temp;
	float	batv;
	char	tm[ 32];
	int		dmy;
	unsigned int	sum;
} SPECTRUM_BUF;

SPECTRUM_BUF	spectrum;
SPECTRUM_BUF	reference;

float		batv;

spi_slave_transaction_t 		spi_slave_trans;
spi_slave_interface_config_t 	spi_slave_cfg;
spi_bus_config_t 				spi_slave_bus;
 
static const uint8_t SPI_SLAVE_CS = 15;
static const uint8_t SPI_SLAVE_CLK = 14;
static const uint8_t SPI_SLAVE_MOSI = 13;
static const uint8_t SPI_SLAVE_MISO = 12;

char	get_sw( void)
{
	char	d;

	d = 0;
	if( digitalRead( IO23) == LOW) d |= 0x01;
	return d;
}

void spi_slave_tans_done(spi_slave_transaction_t* trans)
{
	int	i;

    ESP_ERROR_CHECK(
        spi_slave_queue_trans(HSPI_HOST, &spi_slave_trans, portMAX_DELAY));
        state.kick_data = 1;
}
 
void	spi_init( void)
{
    memset( (char*)&esp32_out, 0, TRANS_SIZE);
    memset( (char*)&esp32_in, 0, TRANS_SIZE);

    spi_slave_trans.length = 8 * TRANS_SIZE;
    spi_slave_trans.rx_buffer = &esp32_in;
    spi_slave_trans.tx_buffer = &esp32_out;
}

void	spi_start( void)
{
    spi_slave_cfg.spics_io_num = SPI_SLAVE_CS;
    spi_slave_cfg.flags = 0;
    spi_slave_cfg.queue_size = 1;
    spi_slave_cfg.mode = SPI_MODE3;
    spi_slave_cfg.post_setup_cb = NULL;
    spi_slave_cfg.post_trans_cb = spi_slave_tans_done;
 
    spi_slave_bus.sclk_io_num = SPI_SLAVE_CLK;
    spi_slave_bus.mosi_io_num = SPI_SLAVE_MOSI;
    spi_slave_bus.miso_io_num = SPI_SLAVE_MISO;
    spi_slave_bus.max_transfer_sz = 8192;
 
    ESP_ERROR_CHECK(
        spi_slave_initialize( HSPI_HOST, &spi_slave_bus, &spi_slave_cfg, 2)); // DMA 2ch
   ESP_ERROR_CHECK(
        spi_slave_queue_trans(HSPI_HOST, &spi_slave_trans, portMAX_DELAY));
}

char	eep_ready( int adr)
{
	if( adr > 0xffff){
		Wire.beginTransmission( EEP_ADRS1);
	} else {
		Wire.beginTransmission( EEP_ADRS0);
	}
	return ( Wire.endTransmission() == 0);	// 0..ok, 1..??, 2..NAK(adr), 3..NAK(data),4..??
}

int	eep_write_page( int adr, char *p, int len)
{
	int		i;
	union	{
		char	b[ 4];
		short	s[2];
		int		l;
	} a;
	int	ret;
	
	a.l = adr;
	if( a.s[1]){
		Wire.beginTransmission( EEP_ADRS1);
	} else {
		Wire.beginTransmission( EEP_ADRS0);
	}
	Wire.write(a.b[1]);
	Wire.write(a.b[0]);
	for( i = 0 ; i < len; i++){
		Wire.write( *p++);
	}
	ret = Wire.endTransmission();
	if( ret == 0){
		while( 1){
			if( a.s[1]){
				if( eep_ready( EEP_ADRS1)) break;
			} else {
				if( eep_ready( EEP_ADRS0)) break;
			}
		}
	}
	return ret;
}

int	eep_read_page( int adr, char *p, int len)
{
	int		i;
	union	{
		char	b[ 4];
		short	s[2];
		int		l;
	} a;
	int	ret;
	
	a.l = adr;
	if( a.s[1]){
		Wire.beginTransmission( EEP_ADRS1);
	} else {
		Wire.beginTransmission( EEP_ADRS0);
	}
	Wire.write(a.b[1]);
	Wire.write(a.b[0]);
	ret = Wire.endTransmission();
	if( ret ){
		return ret;
	}
	if( a.s[1]){
		ret = Wire.requestFrom( EEP_ADRS1, len, 1);
	} else {
		ret = Wire.requestFrom( EEP_ADRS0, len, 1);
	}
	if( ret == len){
		for( i = 0 ; i < len; i++){
			*p++ = Wire.read();
		}
		return 0;
	} else {
		return ret | 0x10000;
	}
}

void	eep_init( void)
{
	Wire.begin( EEP_SDA, EEP_SCL); //Wire.begin(SDA,SCL)
	delay( 10);
}


void	eep_write( int adr, char *p, int sz)
{
	int		i;
	int		len;
	int		ret;
	
	for( i = 0; i < sz; i += EP_WPAGE){
		len = sz - i;
		if( len > EP_WPAGE) len = EP_WPAGE;
		ret = eep_write_page( adr, p, len);
		adr += EP_WPAGE;
		p += EP_WPAGE;
		if( ret) break;
		yield();
	}
}

void	copy_len( char *s, char *d, int len)
{
	int	i;

	for( i = 0 ; i < len; i++){
		*d++ = *s++;
	}
}

int	eep_write_change( int base, int adr, char *p, char *c, int len)
{
	int		j;
	int		chg;
	
	chg = 0;
	for( j = 0; j < len; j++){
		if( p[ adr+j] != c[ adr+j]) chg = 1;
	}
	if( chg){
		yield();
		eep_write_page( base + adr, &c[adr], len);
	}
	adr += len;
	return adr;
}

void	eep_read( int adr, char *p, int size)
{
	int		i;
	int		len;
	int		ret;

	for( i = 0; i < size; i += EP_RPAGE){
		len = size - i;
		if( len > EP_RPAGE) len = EP_RPAGE;
		ret = eep_read_page( adr, p, len);
		if( ret) break;
		adr += EP_RPAGE;
		p += EP_RPAGE;
		yield();
	}
}

void	get_html( char *p)
{
	char	b[ 128];
	int		i;
	char	*s;

	p[ 0] = 0;
	
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"<!DOCTYPE html>\n");	p = strcat( p, b);
	sprintf( b,"<html lang=\"ja\">\n");	p = strcat( p, b);
	sprintf( b,"<head>\n");	p = strcat( p, b);
	sprintf( b,"	<meta charset=\"UTF-8\">\n");	p = strcat( p, b);
	sprintf( b,"	<meta http-equiv=\"X-UA-Compatible\" content=\"IE=edge\">\n");	p = strcat( p, b);
	sprintf( b,"	<meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0\">\n");	p = strcat( p, b);
	sprintf( b,"	<title>ColorCompassWiFi</title>\n");	p = strcat( p, b);
	sprintf( b,"	<link rel=\"stylesheet\" href=\"style.css\" type=\"text/css\">\n");	p = strcat( p, b);
	sprintf( b,"</head>\n");	p = strcat( p, b);
	sprintf( b,"<body>\n");	p = strcat( p, b);
	sprintf( b,"	<header>\n");	p = strcat( p, b);
	sprintf( b,"			ColorCompass WiFi\n");	p = strcat( p, b);
	sprintf( b,"	</header>\n");	p = strcat( p, b);
	sprintf( b,"	<nav id=\"setting\">\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--white\" id=\"button\" >White</p>\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--blue\" id=\"w_set\" >SET</p>\n");	p = strcat( p, b);
	sprintf( b,"		<P  class = \"btn btn--blue\" id=\"w_clr\" >CLR</p>\n");	p = strcat( p, b);
	sprintf( b,"		<P  class = \"btn btn--white\" id=\"button\" >Dark</p>\n");	p = strcat( p, b);
	sprintf( b,"		<P  class = \"btn btn--blue\" id=\"d_set\" >SET</p>\n");	p = strcat( p, b);
	sprintf( b,"		<P  class = \"btn btn--blue\" id=\"d_clr\" >CLR</p>\n");	p = strcat( p, b);
	sprintf( b,"	</nav>\n");	p = strcat( p, b);
	sprintf( b,"	<nav id=\"setting2\">\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--white\" id=\"ref\" >Regist</p>\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--blue\" id=\"ref1\" >#1</p>\n");	p = strcat( p, b);
	sprintf( b,"		<P  class = \"btn btn--blue\" id=\"ref2\" >#2</p>\n");	p = strcat( p, b);
	sprintf( b,"		<P  class = \"btn btn--blue\" id=\"ref3\" >#3</p>\n");	p = strcat( p, b);
	sprintf( b,"		<P  class = \"btn btn--blue\" id=\"ref4\" >#4</p>\n");	p = strcat( p, b);
	sprintf( b,"		<P  class = \"btn btn--blue\" id=\"ref5\" >#5</p>\n");	p = strcat( p, b);
	sprintf( b,"		<P  class = \"btn btn--blue\" id=\"ref6\" >#6</p>\n");	p = strcat( p, b);
	sprintf( b,"	</nav>\n");	p = strcat( p, b);
	sprintf( b,"	<nav id=\"calcuration\">\n");	p = strcat( p, b);
	sprintf( b,"		<p  id=\"calc\">data here</p>\n");	p = strcat( p, b);
	sprintf( b,"	</nav>\n");	p = strcat( p, b);
	sprintf( b,"	<main id=\"graph\">\n");	p = strcat( p, b);
	sprintf( b,"		<canvas width=\"300\" height=\"300\"></canvas>\n");	p = strcat( p, b);
	sprintf( b,"	</main>\n");	p = strcat( p, b);
	sprintf( b,"	<div id=\"command\">\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--white\" id=\"button\">Esposure</p>\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--green\" id=\"up1\">UP</p>\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--green\" id=\"up2\">up</p>\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--green\" id=\"down2\">down</p>\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--green\" id=\"down1\">DOWN</p>\n");	p = strcat( p, b);
	sprintf( b,"	</div>\n");	p = strcat( p, b);
	sprintf( b,"	<div id=\"command2\">\n");	p = strcat( p, b);
	sprintf( b,"		<a  href = \"config.html\" class = \"btn btn--red\" id=\"config\">config</a>\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--green\" id=\"mode\">MODE</p>\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--green\" id=\"disp\">DISP</p>\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--green\" id=\"state\">STATE</p>\n");	p = strcat( p, b);
	sprintf( b,"		<p  class = \"btn btn--red\" id=\"save\">SAVE</p>\n");	p = strcat( p, b);
	sprintf( b,"	</div>\n");	p = strcat( p, b);
	sprintf( b,"	<footer>\n");	p = strcat( p, b);
	sprintf( b,"		<small>\n");	p = strcat( p, b);
	sprintf( b,"			@copyright AT-SYSTEM\n");	p = strcat( p, b);
	sprintf( b,"		</small>\n");	p = strcat( p, b);
	sprintf( b,"	</footer>\n");	p = strcat( p, b);
	sprintf( b,"	<script src=\"index.js\"></script>\n");	p = strcat( p, b);
	sprintf( b,"</body>\n");	p = strcat( p, b);
	sprintf( b,"</html>\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);

}

void handleRoot()
{
     server.send(200, "text/html", html_buf);
}

void	get_style( char *p)
{
 	char	b[ 128];
	int		i;

	p[ 0] = 0;
	
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"  body{\n");	p = strcat( p, b);
	sprintf( b,"    background-color: aqua;\n");	p = strcat( p, b);
	sprintf( b,"    margin:0;\n");	p = strcat( p, b);
	sprintf( b,"    padding: 10px  10px  10px  10px;\n");	p = strcat( p, b);
	sprintf( b,"    user-select: none;\n");	p = strcat( p, b);
	sprintf( b,"  }\n");	p = strcat( p, b);
	sprintf( b,"  header {\n");	p = strcat( p, b);
	sprintf( b,"    background-color: aqua;\n");	p = strcat( p, b);
	sprintf( b,"    color:black;\n");	p = strcat( p, b);
	sprintf( b,"    margin: 0;\n");	p = strcat( p, b);
	sprintf( b,"    padding: 0px 0px 0px 0px;\n");	p = strcat( p, b);
	sprintf( b,"    text-align: center;\n");	p = strcat( p, b);
	sprintf( b,"  }\n");	p = strcat( p, b);
	sprintf( b,"  main {\n");	p = strcat( p, b);
	sprintf( b,"    color:black;\n");	p = strcat( p, b);
	sprintf( b,"    background-color: aqua;\n");	p = strcat( p, b);
	sprintf( b,"    margin: 0px 0px 0px 0px;\n");	p = strcat( p, b);
	sprintf( b,"    padding: 0px 0px 0px 0px;\n");	p = strcat( p, b);
	sprintf( b,"    align-content:fixed;\n");	p = strcat( p, b);
	sprintf( b,"    text-align: center;\n");	p = strcat( p, b);
	sprintf( b,"  }\n");	p = strcat( p, b);
	sprintf( b,"  nav{\n");	p = strcat( p, b);
	sprintf( b,"    color:black;\n");	p = strcat( p, b);
	sprintf( b,"    margin: 0px  0px  0px  0px;\n");	p = strcat( p, b);
	sprintf( b,"    padding: 0px 0px 0px 0px;\n");	p = strcat( p, b);
	sprintf( b,"    text-align: center;\n");	p = strcat( p, b);
	sprintf( b,"    align-content:fixed;\n");	p = strcat( p, b);
	sprintf( b,"  }\n");	p = strcat( p, b);
	sprintf( b,"  div{\n");	p = strcat( p, b);
	sprintf( b,"    background-color: aqua;\n");	p = strcat( p, b);
	sprintf( b,"    margin: 0px  0px  0px  0px;\n");	p = strcat( p, b);
	sprintf( b,"    padding: 0px 0px 0px 0px;\n");	p = strcat( p, b);
	sprintf( b,"    text-align: center;\n");	p = strcat( p, b);
	sprintf( b,"  }\n");	p = strcat( p, b);
	sprintf( b,"  footer{\n");	p = strcat( p, b);
	sprintf( b,"    background-color: aqua;\n");	p = strcat( p, b);
	sprintf( b,"    color:black;\n");	p = strcat( p, b);
	sprintf( b,"    padding: 0px 10px 0px 10px;\n");	p = strcat( p, b);
	sprintf( b,"    text-align: center;\n");	p = strcat( p, b);
	sprintf( b,"  }\n");	p = strcat( p, b);
	sprintf( b,"  html {\n");	p = strcat( p, b);
	sprintf( b,"    -webkit-box-sizing: border-box;\n");	p = strcat( p, b);
	sprintf( b,"    box-sizing: border-box;\n");	p = strcat( p, b);
	sprintf( b,"    font-size: 62.5%;\n");	p = strcat( p, b);
	sprintf( b,"  }\n");	p = strcat( p, b);
	sprintf( b,"  .btn,\n");	p = strcat( p, b);
	sprintf( b,"  a.btn,\n");	p = strcat( p, b);
	sprintf( b,"  button.btn {\n");	p = strcat( p, b);
	sprintf( b,"    font-size: 0.6rem;\n");	p = strcat( p, b);
	sprintf( b,"    font-weight: 200;\n");	p = strcat( p, b);
	sprintf( b,"    line-height: 1.1;\n");	p = strcat( p, b);
	sprintf( b,"    position: relative;\n");	p = strcat( p, b);
	sprintf( b,"    display: inline-block;\n");	p = strcat( p, b);
	sprintf( b,"    padding: 0.5rem 0.5rem;\n");	p = strcat( p, b);
	sprintf( b,"    cursor: pointer;\n");	p = strcat( p, b);
	sprintf( b,"    transition: all 0.3s;\n");	p = strcat( p, b);
	sprintf( b,"    text-align: center;\n");	p = strcat( p, b);
	sprintf( b,"    vertical-align: middle;\n");	p = strcat( p, b);
	sprintf( b,"    text-decoration: none;\n");	p = strcat( p, b);
	sprintf( b,"    letter-spacing: 0.1em;\n");	p = strcat( p, b);
	sprintf( b,"    color: aqua;\n");	p = strcat( p, b);
	sprintf( b,"    border-radius: 0.3rem;\n");	p = strcat( p, b);
	sprintf( b,"  }			\n");	p = strcat( p, b);
	sprintf( b,"  .btn--green,\n");	p = strcat( p, b);
	sprintf( b,"  a.btn--green {\n");	p = strcat( p, b);
	sprintf( b,"    margin: 5px  5px  5px  5px;\n");	p = strcat( p, b);
	sprintf( b,"    color: #ffffff;\n");	p = strcat( p, b);
	sprintf( b,"    background-color: #054612;\n");	p = strcat( p, b);
	sprintf( b,"  }\n");	p = strcat( p, b);
	sprintf( b,"  .btn--blue,\n");	p = strcat( p, b);
	sprintf( b,"  a.btn--blue {\n");	p = strcat( p, b);
	sprintf( b,"    margin: 5px  5px  5px  5px;\n");	p = strcat( p, b);
	sprintf( b,"    color: #ffffff;\n");	p = strcat( p, b);
	sprintf( b,"    background-color: #0000eb;\n");	p = strcat( p, b);
	sprintf( b,"  }\n");	p = strcat( p, b);
	sprintf( b,"  .btn--red,\n");	p = strcat( p, b);
	sprintf( b,"  a.btn--red {\n");	p = strcat( p, b);
	sprintf( b,"    margin: 5px  5px  5px  5px;\n");	p = strcat( p, b);
	sprintf( b,"    color: #ffffff;\n");	p = strcat( p, b);
	sprintf( b,"    background-color: #e70a0a;\n");	p = strcat( p, b);
	sprintf( b,"  }\n");	p = strcat( p, b);
	sprintf( b,"  .btn--white,\n");	p = strcat( p, b);
	sprintf( b,"  a.btn--white {\n");	p = strcat( p, b);
	sprintf( b,"    margin: 5px  5px  5px  5px;\n");	p = strcat( p, b);
	sprintf( b,"    color: #0a0909;\n");	p = strcat( p, b);
	sprintf( b,"    background-color: #72c0ed;\n");	p = strcat( p, b);
	sprintf( b,"  }\n");	p = strcat( p, b);
	sprintf( b,"  .btn--sky,\n");	p = strcat( p, b);
	sprintf( b,"  a.btn--sky {\n");	p = strcat( p, b);
	sprintf( b,"    margin: 5px  5px  5px  5px;\n");	p = strcat( p, b);
	sprintf( b,"    font-size: 1.4rem;\n");	p = strcat( p, b);
	sprintf( b,"    color: #0a0909;\n");	p = strcat( p, b);
	sprintf( b,"    background-color: #72c0ed;\n");	p = strcat( p, b);
	sprintf( b,"  }\n");	p = strcat( p, b);

}
void	handleStyle( void)
{
 	server.send(200, "text/css", style_buf);
}

void	get_javascript( char *p)
{
 	char	b[ 256];
	int		i;

	p[ 0] = 0;

	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"let context = document.getElementsByTagName('canvas')[0].getContext('2d');\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"let gow = context.canvas.width;\n");	p = strcat( p, b);
	sprintf( b,"let goh = context.canvas.height;\n");	p = strcat( p, b);
	sprintf( b,"let giw = gow - 40;\n");	p = strcat( p, b);
	sprintf( b,"let gih = goh - 30;\n");	p = strcat( p, b);
	sprintf( b,"let gix = 25;\n");	p = strcat( p, b);
	sprintf( b,"let giy = 10;\n");	p = strcat( p, b);
	sprintf( b,"let repeat = 1;\n");	p = strcat( p, b);
	sprintf( b,"let sw_data = 0;\n");	p = strcat( p, b);
	sprintf( b,"let	select_no = 0;\n");	p = strcat( p, b);
	sprintf( b,"let message = \"\";\n");	p = strcat( p, b);
	sprintf( b,"let regist = 1;\n");	p = strcat( p, b);
	sprintf( b,"let save_white_dark = 1;\n");	p = strcat( p, b);
	sprintf( b,"let up_down	= 1;\n");	p = strcat( p, b);
	sprintf( b,"let disp_kind = \"data\";\n");	p = strcat( p, b);
	sprintf( b,"let jsonname = \"\";\n");	p = strcat( p, b);
	sprintf( b,"let	tm = \"\";\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"const	sw_up1 = 1;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_up2 = 2;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_down1 = 3;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_down2 = 4;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_state = 5;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_mode = 6;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_disp = 7;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_white_set = 8;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_white_clear = 9;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_dark_set = 10;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_dark_clear = 11;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_ref1	= 12;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_ref2	= 13;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_ref3	= 14;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_ref4	= 15;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_ref5	= 16;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_ref6	= 17;\n");	p = strcat( p, b);
	sprintf( b,"const	sw_save = 18;\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"let w_set = document.getElementById('w_set');\n");	p = strcat( p, b);
	sprintf( b,"let w_clr = document.getElementById('w_clr');\n");	p = strcat( p, b);
	sprintf( b,"let d_set = document.getElementById('d_set');\n");	p = strcat( p, b);
	sprintf( b,"let d_clr = document.getElementById('d_clr');\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"let ref = document.getElementById('ref');\n");	p = strcat( p, b);
	sprintf( b,"let ref1 = document.getElementById('ref1');\n");	p = strcat( p, b);
	sprintf( b,"let ref2 = document.getElementById('ref2');\n");	p = strcat( p, b);
	sprintf( b,"let ref3 = document.getElementById('ref3');\n");	p = strcat( p, b);
	sprintf( b,"let ref4 = document.getElementById('ref4');\n");	p = strcat( p, b);
	sprintf( b,"let ref5 = document.getElementById('ref5');\n");	p = strcat( p, b);
	sprintf( b,"let ref6 = document.getElementById('ref6');\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"let up1 = document.getElementById('up1');\n");	p = strcat( p, b);
	sprintf( b,"let up2 = document.getElementById('up2');\n");	p = strcat( p, b);
	sprintf( b,"let down1 = document.getElementById('down1');\n");	p = strcat( p, b);
	sprintf( b,"let down2 = document.getElementById('down2');\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"let config = document.getElementById('config');\n");	p = strcat( p, b);
	sprintf( b,"let mode = document.getElementById('mode');\n");	p = strcat( p, b);
	sprintf( b,"let disp = document.getElementById('disp');\n");	p = strcat( p, b);
	sprintf( b,"let state = document.getElementById('state');\n");	p = strcat( p, b);
	sprintf( b,"let save = document.getElementById('save');\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"let calcdata = document.getElementById('calc');\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"w_set.onclick = w_setClicked;\n");	p = strcat( p, b);
	sprintf( b,"w_clr.onclick = w_clrClicked;\n");	p = strcat( p, b);
	sprintf( b,"d_set.onclick = d_setClicked;\n");	p = strcat( p, b);
	sprintf( b,"d_clr.onclick = d_clrClicked;\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"ref1.onclick = ref1Clicked;\n");	p = strcat( p, b);
	sprintf( b,"ref2.onclick = ref2Clicked;\n");	p = strcat( p, b);
	sprintf( b,"ref3.onclick = ref3Clicked;\n");	p = strcat( p, b);
	sprintf( b,"ref4.onclick = ref4Clicked;\n");	p = strcat( p, b);
	sprintf( b,"ref5.onclick = ref5Clicked;\n");	p = strcat( p, b);
	sprintf( b,"ref6.onclick = ref6Clicked;\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"up1.onclick	= up1Clicked;\n");	p = strcat( p, b);
	sprintf( b,"up2.onclick	= up2Clicked;\n");	p = strcat( p, b);
	sprintf( b,"down1.onclick	= down1Clicked;\n");	p = strcat( p, b);
	sprintf( b,"down2.onclick	= down2Clicked;\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"mode.onclick = modeClicked;\n");	p = strcat( p, b);
	sprintf( b,"disp.onclick = dispClicked;\n");	p = strcat( p, b);
	sprintf( b,"state.onclick = stateClicked;\n");	p = strcat( p, b);
	sprintf( b,"save.onclick = saveClicked;\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"let	interval;\n");	p = strcat( p, b);
	sprintf( b,"let bar;\n");	p = strcat( p, b);
	sprintf( b,"let rgb_table  = [\n");	p = strcat( p, b);
	sprintf( b,"	\"#000000\",\"#000000\",\"#000000\",\"#000000\",\"#000000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#000000\",\"#000000\",\"#000000\",\"#000000\",\"#000000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#000000\",\"#000000\",\"#000000\",\"#000000\",\"#000000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#000000\",\"#000000\",\"#000000\",\"#000000\",\"#000000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#0F001C\",\"#0F001C\",\"#0F001C\",\"#0F001C\",\"#0F001C\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#16002A\",\"#16002A\",\"#16002A\",\"#16002A\",\"#16002A\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#230043\",\"#230043\",\"#230043\",\"#230043\",\"#230043\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#340065\",\"#340065\",\"#340065\",\"#340065\",\"#340065\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#4F009A\",\"#4F009A\",\"#4F009A\",\"#4F009A\",\"#4F009A\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#6500CD\",\"#6500CD\",\"#6500CD\",\"#6500CD\",\"#6500CD\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#6700DF\",\"#6700DF\",\"#6700DF\",\"#6700DF\",\"#6700DF\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#5700E0\",\"#5700E0\",\"#5700E0\",\"#5700E0\",\"#5700E0\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#1C00DB\",\"#1C00DB\",\"#1C00DB\",\"#1C00DB\",\"#1C00DB\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#0026C6\",\"#0026C6\",\"#0026C6\",\"#0026C6\",\"#0026C6\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#0062A6\",\"#0062A6\",\"#0062A6\",\"#0062A6\",\"#0062A6\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#007E84\",\"#007E84\",\"#007E84\",\"#007E84\",\"#007E84\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#009766\",\"#009766\",\"#009766\",\"#009766\",\"#009766\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#00B141\",\"#00B141\",\"#00B141\",\"#00B141\",\"#00B141\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#00C600\",\"#00C600\",\"#00C600\",\"#00C600\",\"#00C600\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#00D000\",\"#00D000\",\"#00D000\",\"#00D000\",\"#00D000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#00D200\",\"#00D200\",\"#00D200\",\"#00D200\",\"#00D200\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#00CF00\",\"#00CF00\",\"#00CF00\",\"#00CF00\",\"#00CF00\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#78C500\",\"#78C500\",\"#78C500\",\"#78C500\",\"#78C500\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#B2B600\",\"#B2B600\",\"#B2B600\",\"#B2B600\",\"#B2B600\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#D89F00\",\"#D89F00\",\"#D89F00\",\"#D89F00\",\"#D89F00\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#F17E00\",\"#F17E00\",\"#F17E00\",\"#F17E00\",\"#F17E00\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#FE4F00\",\"#FE4F00\",\"#FE4F00\",\"#FE4F00\",\"#FE4F00\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#FF0000\",\"#FF0000\",\"#FF0000\",\"#FF0000\",\"#FF0000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#F30000\",\"#F30000\",\"#F30000\",\"#F30000\",\"#F30000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#DC0000\",\"#DC0000\",\"#DC0000\",\"#DC0000\",\"#DC0000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#C20000\",\"#C20000\",\"#C20000\",\"#C20000\",\"#C20000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#A40000\",\"#A40000\",\"#A40000\",\"#A40000\",\"#A40000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#860000\",\"#860000\",\"#860000\",\"#860000\",\"#860000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#6A0000\",\"#6A0000\",\"#6A0000\",\"#6A0000\",\"#6A0000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#540000\",\"#540000\",\"#540000\",\"#540000\",\"#540000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#400000\",\"#400000\",\"#400000\",\"#400000\",\"#400000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#320000\",\"#320000\",\"#320000\",\"#320000\",\"#320000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#270000\",\"#270000\",\"#270000\",\"#270000\",\"#270000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#1E0000\",\"#1E0000\",\"#1E0000\",\"#1E0000\",\"#1E0000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#170000\",\"#170000\",\"#170000\",\"#170000\",\"#170000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#110000\",\"#110000\",\"#110000\",\"#110000\",\"#110000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#0D0000\",\"#0D0000\",\"#0D0000\",\"#0D0000\",\"#0D0000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#0A0000\",\"#0A0000\",\"#0A0000\",\"#0A0000\",\"#0A0000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#090002\",\"#090002\",\"#090002\",\"#090002\",\"#090002\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#000000\",\"#000000\",\"#000000\",\"#000000\",\"#000000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#000000\",\"#000000\",\"#000000\",\"#000000\",\"#000000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#000000\",\"#000000\",\"#000000\",\"#000000\",\"#000000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#000000\",\"#000000\",\"#000000\",\"#000000\",\"#000000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#000000\",\"#000000\",\"#000000\",\"#000000\",\"#000000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#000000\",\"#000000\",\"#000000\",\"#000000\",\"#000000\",\n");	p = strcat( p, b);
	sprintf( b,"	\"#000000\",\"#000000\",\"#000000\",\"#000000\",\"#000000\",\n");	p = strcat( p, b);
	sprintf( b,"			];\n");	p = strcat( p, b);
	sprintf( b,"  \n");	p = strcat( p, b);
	sprintf( b,"function dummy( data)\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	message = data.msg;\n");	p = strcat( p, b);
	sprintf( b,"	setTimeout(() => {\n");	p = strcat( p, b);
	sprintf( b,"		get_data();\n");	p = strcat( p, b);
	sprintf( b,"	}, 100);\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function up1Clicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( up_down){\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = sw_up1;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function up2Clicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( up_down){\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = sw_up2;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function down1Clicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( up_down){\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = sw_down1;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function down2Clicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( up_down){\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = sw_down2;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function stateClicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( disp_kind != \"ref\"){\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = sw_state;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function saveClicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( save_white_dark){\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = sw_save;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function w_setClicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( save_white_dark){\n");	p = strcat( p, b);
	sprintf( b,"		if(window.confirm( \"white set\")){\n");	p = strcat( p, b);
	sprintf( b,"			sw_data = sw_white_set;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function w_clrClicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( save_white_dark){\n");	p = strcat( p, b);
	sprintf( b,"		if(window.confirm( \"white clear\")){\n");	p = strcat( p, b);
	sprintf( b,"			sw_data = sw_white_clear;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function d_setClicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( save_white_dark){\n");	p = strcat( p, b);
	sprintf( b,"		if(window.confirm( \"dark set\")){\n");	p = strcat( p, b);
	sprintf( b,"			sw_data = sw_dark_set;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function d_clrClicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( save_white_dark){\n");	p = strcat( p, b);
	sprintf( b,"		if(window.confirm( \"dark clear\")){\n");	p = strcat( p, b);
	sprintf( b,"			sw_data = sw_dark_clear;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function modeClicked(){\n");	p = strcat( p, b);
	sprintf( b,"	sw_data = sw_mode;\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function dispClicked(){\n");	p = strcat( p, b);
	sprintf( b,"	sw_data = sw_disp;\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"	\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function ref1Clicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( regist){\n");	p = strcat( p, b);
	sprintf( b,"		if( window.confirm( \"regist #1\")){\n");	p = strcat( p, b);
	sprintf( b,"			sw_data = sw_ref1;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = sw_ref1;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function ref2Clicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( regist){\n");	p = strcat( p, b);
	sprintf( b,"		if( window.confirm( \"regist #2\")){\n");	p = strcat( p, b);
	sprintf( b,"			sw_data = sw_ref2;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = sw_ref2;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function ref3Clicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( regist){\n");	p = strcat( p, b);
	sprintf( b,"		if( window.confirm( \"regist #3\")){\n");	p = strcat( p, b);
	sprintf( b,"			sw_data = sw_ref3;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = sw_ref3;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function ref4Clicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( regist){\n");	p = strcat( p, b);
	sprintf( b,"		if( window.confirm( \"regist #4\")){\n");	p = strcat( p, b);
	sprintf( b,"			sw_data = sw_ref4;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = sw_ref4;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function ref5Clicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( regist){\n");	p = strcat( p, b);
	sprintf( b,"		if( window.confirm( \"regist #5\")){\n");	p = strcat( p, b);
	sprintf( b,"			sw_data = sw_ref5;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = sw_ref5;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function ref6Clicked(){\n");	p = strcat( p, b);
	sprintf( b,"	if( regist){\n");	p = strcat( p, b);
	sprintf( b,"		if( window.confirm( \"regist #6\")){\n");	p = strcat( p, b);
	sprintf( b,"			sw_data = sw_ref6;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = sw_ref6;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function frame( pixel) {\n");	p = strcat( p, b);
	sprintf( b,"	let i;\n");	p = strcat( p, b);
	sprintf( b,"	context.save();\n");	p = strcat( p, b);
	sprintf( b,"	context.beginPath();\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"	blankall();\n");	p = strcat( p, b);
	sprintf( b,"	context.fillStyle	= 'black';\n");	p = strcat( p, b);
	sprintf( b,"	context.fillRect( gix-1, giy-1, 1, gih+2);						// line\n");	p = strcat( p, b);
	sprintf( b,"	context.fillRect( gix+giw, giy-1, 1, gih+2);					// line\n");	p = strcat( p, b);
	sprintf( b,"	context.fillRect( gix-1, giy-1, giw+2, 1);						// line\n");	p = strcat( p, b);
	sprintf( b,"	context.fillRect( gix-1, giy+gih, giw+2, 1);					// line\n");	p = strcat( p, b);
	sprintf( b,"	if( pixel.mode == \"raw\"){\n");	p = strcat( p, b);
	sprintf( b,"		for( i = 0 ; i <= 100; i += 10){\n");	p = strcat( p, b);
	sprintf( b,"			context.fillText( i.toString(), gix - 20, giy + gih + 5 - i*gih/100 );	// left number\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		for( i = 0 ; i <= 100; i += 10){\n");	p = strcat( p, b);
	sprintf( b,"			context.fillText( (i/50).toString(), gix - 20, giy + gih + 5 - i*gih/100 );	// left number\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"	for( i = 350 ; i <= 850; i += 50){\n");	p = strcat( p, b);
	sprintf( b,"		context.fillText( i.toString(), gix-5+(i-340-5) /2, giy + gih + 10 );	// wave number\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"	context.fillStyle	= 'gray';\n");	p = strcat( p, b);
	sprintf( b,"	for( i = 10 ; i <= 90; i += 10){\n");	p = strcat( p, b);
	sprintf( b,"		context.fillRect( gix, giy+gih-i*gih/100, giw, 1);\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"	for( i = 350 ; i <= 850; i += 50){\n");	p = strcat( p, b);
	sprintf( b,"		context.fillRect( gix+(i-340) /2, giy, 1, gih);\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"	context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"	context.restore();\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function blankall() {\n");	p = strcat( p, b);
	sprintf( b,"	context.fillStyle = 'rgba(255,255,192,1)';\n");	p = strcat( p, b);
	sprintf( b,"	context.fillRect( 0, 0, gow, goh);\n");	p = strcat( p, b);
	sprintf( b,"	context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"	context.restore();\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"		\n");	p = strcat( p, b);
	sprintf( b,"function blankgi() {\n");	p = strcat( p, b);
	sprintf( b,"	context.fillStyle = 'rgba(255,255,192,1)';\n");	p = strcat( p, b);
	sprintf( b,"	context.fillRect(gix+1, giy+1, giw-2, gih-2);\n");	p = strcat( p, b);
	sprintf( b,"	context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function get_data(){\n");	p = strcat( p, b);
	sprintf( b,"	fetch( \"./data/data.json\")\n");	p = strcat( p, b);
	sprintf( b,"	.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"	.then( data => graph( data) )\n");	p = strcat( p, b);
	sprintf( b,"	.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"		console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"		context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"		context.fillText( \"data error\", gix+giw/2-50, giy+gih/2 );\n");	p = strcat( p, b);
	sprintf( b,"		context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"		context.restore();\n");	p = strcat( p, b);
	sprintf( b,"		});\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function kick_data(){\n");	p = strcat( p, b);
	sprintf( b,"	fetch( \"./data/start.json\")\n");	p = strcat( p, b);
	sprintf( b,"	.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"	.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"	.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"		console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"		context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"		context.fillText( \"start error\", gix+giw/2-50, giy+gih/2 );\n");	p = strcat( p, b);
	sprintf( b,"		context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"		context.restore();\n");	p = strcat( p, b);
	sprintf( b,"		});\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function save_csv( pixel)\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	let i;\n");	p = strcat( p, b);
	sprintf( b,"	let dt = new Date();\n");	p = strcat( p, b);
	sprintf( b,"	const fileName = dt.getFullYear().toString() + (dt.getMonth()+1).toString() + dt.getDate().toString() + \"_\"+dt.getHours().toString()+dt.getMinutes().toString()+dt.getSeconds().toString()+\".csv\";\n");	p = strcat( p, b);
	sprintf( b,"	var str;\n");	p = strcat( p, b);
	sprintf( b,"	str = \"date,time,temp\";\n");	p = strcat( p, b);
	sprintf( b,"	for( i = 340; i < 852; i+=2){\n");	p = strcat( p, b);
	sprintf( b,"		str += \",\" + i.toString();\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"	str += \"\\n\";\n");	p = strcat( p, b);
	sprintf( b,"	str += dt.getFullYear().toString();\n");	p = strcat( p, b);
	sprintf( b,"	str += \"/\" + (dt.getMonth()+1).toString();\n");	p = strcat( p, b);
	sprintf( b,"	str += \"/\" + dt.getDate().toString();\n");	p = strcat( p, b);
	sprintf( b,"	str += \",\" + dt.getHours().toString();\n");	p = strcat( p, b);
	sprintf( b,"	str += \":\" + dt.getMinutes().toString();\n");	p = strcat( p, b);
	sprintf( b,"	str += \":\" + dt.getSeconds().toString();\n");	p = strcat( p, b);
	sprintf( b,"	str += \",\" + pixel.t.toString();\n");	p = strcat( p, b);
	sprintf( b,"	for( i = 0; i < pixel.n; i++){\n");	p = strcat( p, b);
	sprintf( b,"		str += \",\";\n");	p = strcat( p, b);
	sprintf( b,"		str += pixel.d[ i].toString();\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"	str += \"\\n\";\n");	p = strcat( p, b);
	sprintf( b,"	var blob =new Blob([str],{type:\"text/csv\"});\n");	p = strcat( p, b);
	sprintf( b,"	var link =document.createElement('a');\n");	p = strcat( p, b);
	sprintf( b,"	link.href = URL.createObjectURL(blob);\n");	p = strcat( p, b);
	sprintf( b,"	link.download = fileName;\n");	p = strcat( p, b);
	sprintf( b,"	link.click();\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function save_jason( pixel)\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	const fileName = pixel.month.toString() + pixel.date.toString() + \"_\"+pixel.hour.toString()+pixel.min.toString()+pixel.sec.toString()+\".json\";\n");	p = strcat( p, b);
	sprintf( b,"	const data = JSON.stringify(pixel);\n");	p = strcat( p, b);
	sprintf( b,"	const blob = new Blob([data], { type: 'application/json' });\n");	p = strcat( p, b);
	sprintf( b,"	const link = document.createElement(\"a\");\n");	p = strcat( p, b);
	sprintf( b,"	link.href = window.URL.createObjectURL(blob);\n");	p = strcat( p, b);
	sprintf( b,"	link.download = fileName;\n");	p = strcat( p, b);
	sprintf( b,"	link.click();\n");	p = strcat( p, b);
	sprintf( b,"	\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function errorFunc( error )\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	var errorMessage = {\n");	p = strcat( p, b);
	sprintf( b,"		0: \"no reason\" ,\n");	p = strcat( p, b);
	sprintf( b,"		1: \"cannot get permission\" ,\n");	p = strcat( p, b);
	sprintf( b,"		2: \"week wave\" ,\n");	p = strcat( p, b);
	sprintf( b,"		3: \"time out\" ,\n");	p = strcat( p, b);
	sprintf( b,"	} ;\n");	p = strcat( p, b);
	sprintf( b,"	alert( errorMessage[error.code] ) ;\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function set_ref_button( no)\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	ref1.style.backgroundColor = \"#3f4811\";\n");	p = strcat( p, b);
	sprintf( b,"	ref2.style.backgroundColor = \"#3f4811\";\n");	p = strcat( p, b);
	sprintf( b,"	ref3.style.backgroundColor = \"#3f4811\";\n");	p = strcat( p, b);
	sprintf( b,"	ref4.style.backgroundColor = \"#3f4811\";\n");	p = strcat( p, b);
	sprintf( b,"	ref5.style.backgroundColor = \"#3f4811\";\n");	p = strcat( p, b);
	sprintf( b,"	ref6.style.backgroundColor = \"#3f4811\";\n");	p = strcat( p, b);
	sprintf( b,"	if( regist == 0){\n");	p = strcat( p, b);
	sprintf( b,"		if( no ==1) {\n");	p = strcat( p, b);
	sprintf( b,"			ref1.style.backgroundColor = \"#3fb811\";\n");	p = strcat( p, b);
	sprintf( b,"		} else if( no == 2){\n");	p = strcat( p, b);
	sprintf( b,"			ref2.style.backgroundColor = \"#3fb811\";\n");	p = strcat( p, b);
	sprintf( b,"		} else if( no == 3){\n");	p = strcat( p, b);
	sprintf( b,"			ref3.style.backgroundColor = \"#3fb811\";\n");	p = strcat( p, b);
	sprintf( b,"		} else if( no == 4){\n");	p = strcat( p, b);
	sprintf( b,"			ref4.style.backgroundColor = \"#3fb811\";\n");	p = strcat( p, b);
	sprintf( b,"		} else if( no == 5){\n");	p = strcat( p, b);
	sprintf( b,"			ref5.style.backgroundColor = \"#3fb811\";\n");	p = strcat( p, b);
	sprintf( b,"		} else if( no == 6){\n");	p = strcat( p, b);
	sprintf( b,"			ref6.style.backgroundColor = \"#3fb811\";\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function set_swd_button( ena)\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	if( ena){\n");	p = strcat( p, b);
	sprintf( b,"		save.style.backgroundColor = \"#ff0000\";\n");	p = strcat( p, b);
	sprintf( b,"		w_set.style.backgroundColor = \"#0000ff\";\n");	p = strcat( p, b);
	sprintf( b,"		w_clr.style.backgroundColor = \"#0000ff\";\n");	p = strcat( p, b);
	sprintf( b,"		d_set.style.backgroundColor = \"#0000ff\";\n");	p = strcat( p, b);
	sprintf( b,"		d_clr.style.backgroundColor = \"#0000ff\";\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		save.style.backgroundColor = \"#808080\";\n");	p = strcat( p, b);
	sprintf( b,"		w_set.style.backgroundColor = \"#808080\";\n");	p = strcat( p, b);
	sprintf( b,"		w_clr.style.backgroundColor = \"#808080\";\n");	p = strcat( p, b);
	sprintf( b,"		d_set.style.backgroundColor = \"#808080\";\n");	p = strcat( p, b);
	sprintf( b,"		d_clr.style.backgroundColor = \"#808080\";\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function set_ud_button( ena)\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	if( ena){\n");	p = strcat( p, b);
	sprintf( b,"		up1.style.backgroundColor = \"#10cf10\";\n");	p = strcat( p, b);
	sprintf( b,"		up2.style.backgroundColor = \"#10cf10\";\n");	p = strcat( p, b);
	sprintf( b,"		down1.style.backgroundColor = \"#10cf10\";\n");	p = strcat( p, b);
	sprintf( b,"		down2.style.backgroundColor = \"#10cf10\";\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		up1.style.backgroundColor = \"#808080\";\n");	p = strcat( p, b);
	sprintf( b,"		up2.style.backgroundColor = \"#808080\";\n");	p = strcat( p, b);
	sprintf( b,"		down1.style.backgroundColor = \"#808080\";\n");	p = strcat( p, b);
	sprintf( b,"		down2.style.backgroundColor = \"#808080\";\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function set_disp_button( kind)\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	if( kind == \"data\"){\n");	p = strcat( p, b);
	sprintf( b,"		disp.style.backgroundColor = \"#10cf10\";\n");	p = strcat( p, b);
	sprintf( b,"		disp.textContent = \"DATA\";\n");	p = strcat( p, b);
	sprintf( b,"	} else if( kind == \"comp\"){\n");	p = strcat( p, b);
	sprintf( b,"		disp.style.backgroundColor = \"#808010\";\n");	p = strcat( p, b);
	sprintf( b,"		disp.textContent = \"COMP\";\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		disp.style.backgroundColor = \"#108080\";\n");	p = strcat( p, b);
	sprintf( b,"		disp.textContent = \"REF+\";\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function set_state_button( kind)\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	if( kind == \"ref\"){\n");	p = strcat( p, b);
	sprintf( b,"		state.style.backgroundColor = \"#808080\";\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		state.style.backgroundColor = \"#108010\";\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function set_mode_button( kind)\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	if( kind == \"raw\"){\n");	p = strcat( p, b);
	sprintf( b,"		mode.style.backgroundColor = \"#408080\";\n");	p = strcat( p, b);
	sprintf( b,"		mode.textContent = \"R A W\";\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		mode.style.backgroundColor = \"#808040\";\n");	p = strcat( p, b);
	sprintf( b,"		mode.textContent = \"reflect\";\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"function graph( pixel) {\n");	p = strcat( p, b);
	sprintf( b,"	let i;\n");	p = strcat( p, b);
	sprintf( b,"	let h;\n");	p = strcat( p, b);
	sprintf( b,"	let d;\n");	p = strcat( p, b);
	sprintf( b,"	let no;\n");	p = strcat( p, b);
	sprintf( b,"	let n;\n");	p = strcat( p, b);
	sprintf( b,"	let date;\n");	p = strcat( p, b);
	sprintf( b,"	let time;\n");	p = strcat( p, b);
	sprintf( b,"	let xs;\n");	p = strcat( p, b);
	sprintf( b,"	let ys;\n");	p = strcat( p, b);
	sprintf( b,"	let old;\n");	p = strcat( p, b);
	sprintf( b,"	let dt = new Date();\n");	p = strcat( p, b);
	sprintf( b,"	frame( pixel);\n");	p = strcat( p, b);
	sprintf( b,"	if( pixel.n > 0){\n");	p = strcat( p, b);
	sprintf( b,"		date = dt.getFullYear().toString()+\"/\"+(dt.getMonth()+1).toString()+\"/\"+dt.getDate().toString();\n");	p = strcat( p, b);
	sprintf( b,"		time = dt.getHours().toString()+\":\"+dt.getMinutes().toString()+\":\"+dt.getSeconds().toString();\n");	p = strcat( p, b);
	sprintf( b,"		tm = date + \" \" + time;\n");	p = strcat( p, b);
	sprintf( b,"		if( pixel.disp == \"data\"){\n");	p = strcat( p, b);
	sprintf( b,"			ref.textContent = \"regist\";\n");	p = strcat( p, b);
	sprintf( b,"			regist = 1;\n");	p = strcat( p, b);
	sprintf( b,"			save_white_dark = 1;\n");	p = strcat( p, b);
	sprintf( b,"		} else {\n");	p = strcat( p, b);
	sprintf( b,"			ref.textContent = \"select\";\n");	p = strcat( p, b);
	sprintf( b,"			regist = 0;\n");	p = strcat( p, b);
	sprintf( b,"			save_white_dark = 0;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"		set_ref_button( select_no);\n");	p = strcat( p, b);
	sprintf( b,"		set_swd_button( save_white_dark);\n");	p = strcat( p, b);
	sprintf( b,"		if( pixel.disp == \"ref\"){\n");	p = strcat( p, b);
	sprintf( b,"			up_down = 0;\n");	p = strcat( p, b);
	sprintf( b,"		} else {\n");	p = strcat( p, b);
	sprintf( b,"			up_down = 1;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"		set_ud_button( up_down);\n");	p = strcat( p, b);
	sprintf( b,"		set_disp_button( pixel.disp);\n");	p = strcat( p, b);
	sprintf( b,"		set_state_button( pixel.disp);\n");	p = strcat( p, b);
	sprintf( b,"		set_mode_button( pixel.mode);\n");	p = strcat( p, b);
	sprintf( b,"		disp_kind = pixel.disp;\n");	p = strcat( p, b);
	sprintf( b,"		select_no = pixel.c;\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"		if( pixel.n < 256){\n");	p = strcat( p, b);
	sprintf( b,"			n = pixel.n;\n");	p = strcat( p, b);
	sprintf( b,"		} else {\n");	p = strcat( p, b);
	sprintf( b,"			n = 256;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"		old = 0;\n");	p = strcat( p, b);
	sprintf( b,"		for(i = 0 ; i < n; i++){\n");	p = strcat( p, b);
	sprintf( b,"			if( pixel.mode == \"raw\"){\n");	p = strcat( p, b);
	sprintf( b,"				d = pixel.r[ i]*gih/65535;\n");	p = strcat( p, b);
	sprintf( b,"			} else {\n");	p = strcat( p, b);
	sprintf( b,"				d = pixel.r[ i]*gih/2;\n");	p = strcat( p, b);
	sprintf( b,"			}\n");	p = strcat( p, b);
	sprintf( b,"			if( d < gih){\n");	p = strcat( p, b);
	sprintf( b,"				h = d;\n");	p = strcat( p, b);
	sprintf( b,"			} else {\n");	p = strcat( p, b);
	sprintf( b,"				h = gih;\n");	p = strcat( p, b);
	sprintf( b,"			}\n");	p = strcat( p, b);
	sprintf( b,"			if( pixel.disp == \"comp\"){\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle = rgb_table[0];\n");	p = strcat( p, b);
	sprintf( b,"				if( h > old){\n");	p = strcat( p, b);
	sprintf( b,"					context.fillRect( gix+i, giy+gih-h, 1, h - old);\n");	p = strcat( p, b);
	sprintf( b,"				} else if( h < old){\n");	p = strcat( p, b);
	sprintf( b,"					context.fillRect( gix+i-1, giy+gih-old, 1, old - h -1);\n");	p = strcat( p, b);
	sprintf( b,"					context.fillRect( gix+i, giy+gih-h, 1, 1);\n");	p = strcat( p, b);
	sprintf( b,"				} else {\n");	p = strcat( p, b);
	sprintf( b,"					context.fillRect( gix+i, giy+gih-h, 1, 1);\n");	p = strcat( p, b);
	sprintf( b,"				}\n");	p = strcat( p, b);
	sprintf( b,"				old = h;\n");	p = strcat( p, b);
	sprintf( b,"			} else if( pixel.disp == \"ref\"){\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle = rgb_table[i];\n");	p = strcat( p, b);
	sprintf( b,"				context.fillRect( gix+i, giy+gih-h, 1, h);\n");	p = strcat( p, b);
	sprintf( b,"			}\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"		old = 0;\n");	p = strcat( p, b);
	sprintf( b,"		for(i = 0 ; i < n; i++){\n");	p = strcat( p, b);
	sprintf( b,"			context.fillStyle = rgb_table[i];\n");	p = strcat( p, b);
	sprintf( b,"			if( pixel.mode == \"raw\"){\n");	p = strcat( p, b);
	sprintf( b,"				d = pixel.d[ i]*gih/65535;\n");	p = strcat( p, b);
	sprintf( b,"			} else {\n");	p = strcat( p, b);
	sprintf( b,"				d = pixel.d[ i]*gih/2;\n");	p = strcat( p, b);
	sprintf( b,"			}\n");	p = strcat( p, b);
	sprintf( b,"			if( d < gih){\n");	p = strcat( p, b);
	sprintf( b,"				h = d;\n");	p = strcat( p, b);
	sprintf( b,"			} else {\n");	p = strcat( p, b);
	sprintf( b,"				h = gih;\n");	p = strcat( p, b);
	sprintf( b,"			}\n");	p = strcat( p, b);
	sprintf( b,"			if( pixel.disp == \"comp\"){\n");	p = strcat( p, b);
	sprintf( b,"				if( h > old){\n");	p = strcat( p, b);
	sprintf( b,"					context.fillRect( gix+i, giy+gih-h, 1, h - old);\n");	p = strcat( p, b);
	sprintf( b,"				} else if( h < old) {\n");	p = strcat( p, b);
	sprintf( b,"					context.fillRect( gix+i-1, giy+gih-old, 1, old - h-1);\n");	p = strcat( p, b);
	sprintf( b,"					context.fillRect( gix+i, giy+gih-h, 1, 1);\n");	p = strcat( p, b);
	sprintf( b,"				} else {\n");	p = strcat( p, b);
	sprintf( b,"					context.fillRect( gix+i, giy+gih-h, 1, 1);\n");	p = strcat( p, b);
	sprintf( b,"				}\n");	p = strcat( p, b);
	sprintf( b,"				old = h;\n");	p = strcat( p, b);
	sprintf( b,"			} else if( pixel.disp == \"data\"){\n");	p = strcat( p, b);
	sprintf( b,"				context.fillRect( gix+i, giy+gih-h, 1, h);\n");	p = strcat( p, b);
	sprintf( b,"			}\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"		if( pixel.save){\n");	p = strcat( p, b);
	sprintf( b,"			save_csv( pixel);\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"		repeat = pixel.repeat;\n");	p = strcat( p, b);
	sprintf( b,"		xs = 5;\n");	p = strcat( p, b);
	sprintf( b,"		ys = 10;\n");	p = strcat( p, b);
	sprintf( b,"		context.fillStyle = 'blue';\n");	p = strcat( p, b);
	sprintf( b,"		if( pixel.disp == \"data\"){\n");	p = strcat( p, b);
	sprintf( b,"			context.fillText( \"Exposure(us)=\"+pixel.e.toString(), gix + xs, giy + ys );	ys += 14;\n");	p = strcat( p, b);
	sprintf( b,"		} else if( pixel.disp == \"comp\"){\n");	p = strcat( p, b);
	sprintf( b,"			context.fillText( \"Exposure(us)=\"+pixel.e.toString()+ \"/\"+pixel.f.toString(), gix + xs, giy + ys );	ys += 14;\n");	p = strcat( p, b);
	sprintf( b,"		} else {\n");	p = strcat( p, b);
	sprintf( b,"			context.fillText( \"registered=\"+pixel.tm, gix + xs, giy + ys );	ys += 14;\n");	p = strcat( p, b);
	sprintf( b,"			context.fillText( \"Exposure(us)=\"+pixel.f.toString(), gix + xs, giy + ys );	ys += 14;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"		context.fillText( \"average num =\"+pixel.a.toString(), gix + xs, giy + ys );	ys += 14;\n");	p = strcat( p, b);
	sprintf( b,"		context.fillText( \"temp        =\"+pixel.t.toString(), gix + xs, giy + ys );	ys += 14;\n");	p = strcat( p, b);
	sprintf( b,"		context.fillText( \"bat voltage =\"+pixel.v.toString(), gix + xs, giy + ys );	ys += 14;\n");	p = strcat( p, b);
	sprintf( b,"		if( repeat){\n");	p = strcat( p, b);
	sprintf( b,"			context.fillText( \"state   =monitoring\", gix + xs, giy + ys );ys += 14;\n");	p = strcat( p, b);
	sprintf( b,"		} else {\n");	p = strcat( p, b);
	sprintf( b,"			context.fillText( \"state   =stopped\", gix + xs, giy + ys );ys += 14;\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		context.fillStyle = 'red';\n");	p = strcat( p, b);
	sprintf( b,"		context.fillText( \" no data \", gix +55, giy + 30 );	// no data\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"	context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"	context.restore();\n");	p = strcat( p, b);
	sprintf( b,"	if( message != \"\"){\n");	p = strcat( p, b);
	sprintf( b,"		context.fillStyle = 'blue';\n");	p = strcat( p, b);
	sprintf( b,"		context.fillText( message, gix+giw/2-50, giy+gih/2);\n");	p = strcat( p, b);
	sprintf( b,"		next_data(500);\n");	p = strcat( p, b);
	sprintf( b,"		message = \"\";\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		next_data(100);\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"	calcdata.innerText = pixel.msg;\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function	next_data(  dly)\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	if( sw_data == 0){\n");	p = strcat( p, b);
	sprintf( b,"		if( repeat){\n");	p = strcat( p, b);
	sprintf( b,"			setTimeout(() => {\n");	p = strcat( p, b);
	sprintf( b,"				get_data();\n");	p = strcat( p, b);
	sprintf( b,"			}, dly);\n");	p = strcat( p, b);
	sprintf( b,"		} else {\n");	p = strcat( p, b);
	sprintf( b,"			sleep_a_little();\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"	} else {\n");	p = strcat( p, b);
	sprintf( b,"		if( sw_data == sw_up1){\n");	p = strcat( p, b);
	sprintf( b,"			fetch( \"./data/up1.json\")\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"up1 error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_up2){\n");	p = strcat( p, b);
	sprintf( b,"			fetch( \"./data/up2.json\")\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"up2 error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_down1){\n");	p = strcat( p, b);
	sprintf( b,"			fetch( \"./data/down1.json\")\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"down1 error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_down2){\n");	p = strcat( p, b);
	sprintf( b,"			fetch( \"./data/down2.json\")\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"down2 error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_state){\n");	p = strcat( p, b);
	sprintf( b,"			fetch( \"./data/repeat.json\")\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"repeat error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_save){\n");	p = strcat( p, b);
	sprintf( b,"			fetch( \"./data/save.json\")\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"save error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_mode){\n");	p = strcat( p, b);
	sprintf( b,"			fetch( \"./data/mode.json\")\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"dark clear error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_disp){\n");	p = strcat( p, b);
	sprintf( b,"			fetch( \"./data/disp.json\")\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"disp error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_white_set){\n");	p = strcat( p, b);
	sprintf( b,"			fetch( \"./data/wset.json\")\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"white set error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_white_clear){\n");	p = strcat( p, b);
	sprintf( b,"			fetch( \"./data/wclear.json\")\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"white clear error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_dark_set){\n");	p = strcat( p, b);
	sprintf( b,"			fetch( \"./data/dset.json\")\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"dark set error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_dark_clear){\n");	p = strcat( p, b);
	sprintf( b,"			fetch( \"./data/dclear.json\")\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"dark clear error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_ref1){\n");	p = strcat( p, b);
	sprintf( b,"			jsonname = \"./data/ref1.json?s=\"+tm;\n");	p = strcat( p, b);
	sprintf( b,"			fetch( jsonname)\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"ref #1 error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_ref2){\n");	p = strcat( p, b);
	sprintf( b,"			jsonname = \"./data/ref2.json?s=\"+tm;\n");	p = strcat( p, b);
	sprintf( b,"			fetch( jsonname)\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"ref #2 error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_ref3){\n");	p = strcat( p, b);
	sprintf( b,"			jsonname = \"./data/ref3.json?s=\"+tm;\n");	p = strcat( p, b);
	sprintf( b,"			fetch( jsonname)\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"ref #3 error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_ref4){\n");	p = strcat( p, b);
	sprintf( b,"			jsonname = \"./data/ref4.json?s=\"+tm;\n");	p = strcat( p, b);
	sprintf( b,"			fetch( jsonname)\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"ref #4 error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_ref5){\n");	p = strcat( p, b);
	sprintf( b,"			jsonname = \"./data/ref5.json?s=\"+tm;\n");	p = strcat( p, b);
	sprintf( b,"			fetch( jsonname)\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"ref #5 error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		} else if( sw_data == sw_ref6){\n");	p = strcat( p, b);
	sprintf( b,"			jsonname = \"./data/ref6.json?s=\"+tm;\n");	p = strcat( p, b);
	sprintf( b,"			fetch( jsonname)\n");	p = strcat( p, b);
	sprintf( b,"			.then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"			.then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"			.catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"				console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"				context.fillStyle	= 'red';\n");	p = strcat( p, b);
	sprintf( b,"				context.fillText( \"ref #6 error\", gix+giw/2-50, giy+gih/2 );	// error message\n");	p = strcat( p, b);
	sprintf( b,"				context.stroke();\n");	p = strcat( p, b);
	sprintf( b,"				context.restore();\n");	p = strcat( p, b);
	sprintf( b,"			});\n");	p = strcat( p, b);
	sprintf( b,"		}\n");	p = strcat( p, b);
	sprintf( b,"		sw_data = 0;\n");	p = strcat( p, b);
	sprintf( b,"	}\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function	sleep_a_little( )\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	setTimeout(() => {\n");	p = strcat( p, b);
	sprintf( b,"		next_data( 500);\n");	p = strcat( p, b);
	sprintf( b,"	}, 100);\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function	start_data( )\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"	setTimeout(() => {\n");	p = strcat( p, b);
	sprintf( b,"		kick_data();\n");	p = strcat( p, b);
	sprintf( b,"	}, 500);\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"start_data();\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);

}

void	handleJavaScript( void)
{
     server.send(200, "text/html", js_buf);
}

void	get_html_config( char *p)
{
	char	b[ 256];

	p[ 0] = 0;
	
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"<!DOCTYPE html>\n");	p = strcat( p, b);
	sprintf( b,"<html lang=\"ja\">\n");	p = strcat( p, b);
	sprintf( b,"<head>\n");	p = strcat( p, b);
	sprintf( b,"	<meta charset=\"UTF-8\">\n");	p = strcat( p, b);
	sprintf( b,"	<meta http-equiv=\"X-UA-Compatible\" content=\"IE=edge\">\n");	p = strcat( p, b);
	sprintf( b,"	<meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0\">\n");	p = strcat( p, b);
	sprintf( b,"	<title>ColorCompassWiFi</title>\n");	p = strcat( p, b);
	sprintf( b,"	<link rel=\"stylesheet\" href=\"style.css\" type=\"text/css\">\n");	p = strcat( p, b);
	sprintf( b,"</head>\n");	p = strcat( p, b);
	sprintf( b,"<body>\n");	p = strcat( p, b);
	sprintf( b,"	<header>\n");	p = strcat( p, b);
	sprintf( b,"			ColorCompass WiFi Configuration\n");	p = strcat( p, b);
	sprintf( b,"	</header>\n");	p = strcat( p, b);
	sprintf( b,"	<nav id=\"command\">\n");	p = strcat( p, b);
	sprintf( b,"		<div class=\"form-ssid\">\n");	p = strcat( p, b);
	sprintf( b,"			<label for=\"wifissid\">wifi ssid: </label>\n");	p = strcat( p, b);
	sprintf( b,"			<input type=\"text\" name=\"ssid\" id=\"ssid\">\n");	p = strcat( p, b);
	sprintf( b,"			<p  class = \"btn btn--white\" id=\"button1\" >set</p>\n");	p = strcat( p, b);
	sprintf( b,"		</div>\n");	p = strcat( p, b);
	sprintf( b,"		<div class=\"form-key\">\n");	p = strcat( p, b);
	sprintf( b,"			<label for=\"wifikey\">wifi key : </label>\n");	p = strcat( p, b);
	sprintf( b,"			<input type=\"text\" name=\"key\" id=\"key\">\n");	p = strcat( p, b);
	sprintf( b,"			<p  class = \"btn btn--white\" id=\"button2\" >set</p>\n");	p = strcat( p, b);
	sprintf( b,"		</div>\n");	p = strcat( p, b);
	sprintf( b,"		<div class=\"form-ip\">\n");	p = strcat( p, b);
	sprintf( b,"			<label for=\"wifiip\">wifi ipad: </label>\n");	p = strcat( p, b);
	sprintf( b,"			<input type=\"text\" name=\"ip\" id=\"ip\">\n");	p = strcat( p, b);
	sprintf( b,"			<p  class = \"btn btn--white\" id=\"button3\" >set</p>\n");	p = strcat( p, b);
	sprintf( b,"		</div>\n");	p = strcat( p, b);
	sprintf( b,"		<div class=\"form-average\">\n");	p = strcat( p, b);
	sprintf( b,"			<label for=\"averagenum\">average  : </label>\n");	p = strcat( p, b);
	sprintf( b,"			<input type=\"text\" name=\"average\" id=\"average\">\n");	p = strcat( p, b);
	sprintf( b,"			<p  class = \"btn btn--white\" id=\"button4\" >set</p>\n");	p = strcat( p, b);
	sprintf( b,"		</div>\n");	p = strcat( p, b);
	sprintf( b,"		<div class=\"form-wave1\">\n");	p = strcat( p, b);
	sprintf( b,"			<label for=\"wave1\">wave1(nm)\"  : </label>\n");	p = strcat( p, b);
	sprintf( b,"			<input type=\"text\" name=\"wave1\" id=\"wave1\">\n");	p = strcat( p, b);
	sprintf( b,"			<p  class = \"btn btn--white\" id=\"button5\" >set</p>\n");	p = strcat( p, b);
	sprintf( b,"		</div>\n");	p = strcat( p, b);
	sprintf( b,"		<div class=\"form-wave2\">\n");	p = strcat( p, b);
	sprintf( b,"			<label for=\"wave2\">wave2(nm)\"  : </label>\n");	p = strcat( p, b);
	sprintf( b,"			<input type=\"text\" name=\"wave2\" id=\"wave2\">\n");	p = strcat( p, b);
	sprintf( b,"			<p  class = \"btn btn--white\" id=\"button6\" >set</p>\n");	p = strcat( p, b);
	sprintf( b,"		</div>\n");	p = strcat( p, b);
	sprintf( b,"		<div id=\"fver\">\n");	p = strcat( p, b);
	sprintf( b,"			<p  id=\"fver\">firm version</p>\n");	p = strcat( p, b);
	sprintf( b,"		</div>\n");	p = strcat( p, b);
	sprintf( b,"	<footer>\n");	p = strcat( p, b);
	sprintf( b,"		<small>\n");	p = strcat( p, b);
	sprintf( b,"			@copyright AT-SYSTEM\n");	p = strcat( p, b);
	sprintf( b,"		</small>\n");	p = strcat( p, b);
	sprintf( b,"	</footer>\n");	p = strcat( p, b);
	sprintf( b,"	<script src=\"config.js\"></script>\n");	p = strcat( p, b);
	sprintf( b,"</body>\n");	p = strcat( p, b);
	sprintf( b,"</html>\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);

}

void	handleHtmlConfig( void)
{
     server.send(200, "text/html", config_html_buf);
}

void	get_js_config( char *p)
{
	char	b[ 256];

	p[ 0] = 0;

	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"let ssid    = document.getElementById('ssid');\n");	p = strcat( p, b);
	sprintf( b,"let key     = document.getElementById('key');\n");	p = strcat( p, b);
	sprintf( b,"let ip      = document.getElementById('ip');\n");	p = strcat( p, b);
	sprintf( b,"let average = document.getElementById('average');\n");	p = strcat( p, b);
	sprintf( b,"let wave1 = document.getElementById('wave1');\n");	p = strcat( p, b);
	sprintf( b,"let wave2 = document.getElementById('wave2');\n");	p = strcat( p, b);
	sprintf( b,"let set1    = document.getElementById('button1');\n");	p = strcat( p, b);
	sprintf( b,"let set2    = document.getElementById('button2');\n");	p = strcat( p, b);
	sprintf( b,"let set3    = document.getElementById('button3');\n");	p = strcat( p, b);
	sprintf( b,"let set4    = document.getElementById('button4');\n");	p = strcat( p, b);
	sprintf( b,"let set5    = document.getElementById('button5');\n");	p = strcat( p, b);
	sprintf( b,"let set6    = document.getElementById('button6');\n");	p = strcat( p, b);
	sprintf( b,"let fver    = document.getElementById('fver');\n");	p = strcat( p, b);
	sprintf( b,"let filename;\n");	p = strcat( p, b);
	sprintf( b,"let dasoku  = \"colorcompass\";\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"set1.onclick = set1Clicked;\n");	p = strcat( p, b);
	sprintf( b,"set2.onclick = set2Clicked;\n");	p = strcat( p, b);
	sprintf( b,"set3.onclick = set3Clicked;\n");	p = strcat( p, b);
	sprintf( b,"set4.onclick = set4Clicked;\n");	p = strcat( p, b);
	sprintf( b,"set5.onclick = set5Clicked;\n");	p = strcat( p, b);
	sprintf( b,"set6.onclick = set6Clicked;\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function    dummy( data)\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"function set1Clicked()\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"    if( window.confirm( \"set ssid=\"+ssid.value)){\n");	p = strcat( p, b);
	sprintf( b,"        filename = \"./data/confset1.json?s=\"+ssid.value+dasoku;\n");	p = strcat( p, b);
	sprintf( b,"        fetch( filename)\n");	p = strcat( p, b);
	sprintf( b,"        .then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"        .then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"        .catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"            console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"        });\n");	p = strcat( p, b);
	sprintf( b,"    }\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function set2Clicked()\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"    if( window.confirm( \"set password =\"+key.value)){\n");	p = strcat( p, b);
	sprintf( b,"        filename = \"./data/confset2.json?s=\"+key.value+dasoku;\n");	p = strcat( p, b);
	sprintf( b,"        fetch( filename)\n");	p = strcat( p, b);
	sprintf( b,"        .then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"        .then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"        .catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"            console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"        });\n");	p = strcat( p, b);
	sprintf( b,"    }\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function set3Clicked()\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"    if( window.confirm( \"set ip address =\"+ip.value)){\n");	p = strcat( p, b);
	sprintf( b,"        filename = \"./data/confset3.json?s=\"+ip.value+dasoku;\n");	p = strcat( p, b);
	sprintf( b,"        fetch( filename)\n");	p = strcat( p, b);
	sprintf( b,"        .then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"        .then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"        .catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"            console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"        });\n");	p = strcat( p, b);
	sprintf( b,"    }\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function set4Clicked()\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"    if( window.confirm( \"set average =\"+average.value)){\n");	p = strcat( p, b);
	sprintf( b,"        filename = \"./data/confset4.json?s=\"+average.value;\n");	p = strcat( p, b);
	sprintf( b,"        fetch( filename)\n");	p = strcat( p, b);
	sprintf( b,"        .then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"        .then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"        .catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"            console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"        });\n");	p = strcat( p, b);
	sprintf( b,"    }\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function set5Clicked()\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"    if( window.confirm( \"set wave1=\"+wave1.value)){\n");	p = strcat( p, b);
	sprintf( b,"        filename = \"./data/confset5.json?s=\"+wave1.value;\n");	p = strcat( p, b);
	sprintf( b,"        fetch( filename)\n");	p = strcat( p, b);
	sprintf( b,"        .then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"        .then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"        .catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"            console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"        });\n");	p = strcat( p, b);
	sprintf( b,"    }\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function set6Clicked()\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"    if( window.confirm( \"set wave2 = \"+wave2.value)){\n");	p = strcat( p, b);
	sprintf( b,"        filename = \"./data/confset6.json?s=\"+wave2.value;\n");	p = strcat( p, b);
	sprintf( b,"        fetch( filename)\n");	p = strcat( p, b);
	sprintf( b,"        .then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"        .then( data => dummy( data) )\n");	p = strcat( p, b);
	sprintf( b,"        .catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"            console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"        });\n");	p = strcat( p, b);
	sprintf( b,"    }\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function disp( data)\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"    ssid.value      = data.ssid;\n");	p = strcat( p, b);
	sprintf( b,"    key.value       = data.key;\n");	p = strcat( p, b);
	sprintf( b,"    ip.value        = data.ip;\n");	p = strcat( p, b);
	sprintf( b,"    average.value   = data.average;\n");	p = strcat( p, b);
	sprintf( b,"    wave1.value     = data.wave1;\n");	p = strcat( p, b);
	sprintf( b,"    wave2.value     = data.wave2;\n");	p = strcat( p, b);
	sprintf( b,"    fver.innerText  = data.fver;\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"function getConfig()\n");	p = strcat( p, b);
	sprintf( b,"{\n");	p = strcat( p, b);
	sprintf( b,"    fetch( \"./data/config.json\")\n");	p = strcat( p, b);
	sprintf( b,"    .then( response => response.json())\n");	p = strcat( p, b);
	sprintf( b,"    .then( data => disp( data) )\n");	p = strcat( p, b);
	sprintf( b,"    .catch((reason) => {\n");	p = strcat( p, b);
	sprintf( b,"        console.log( reason);\n");	p = strcat( p, b);
	sprintf( b,"    });\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"}\n");	p = strcat( p, b);
	sprintf( b,"\n");	p = strcat( p, b);
	sprintf( b,"getConfig();\n");	p = strcat( p, b);

}

void	handleJsConfig( void)
{
     server.send(200, "text/html", config_js_buf);
}

void	handleConfigJson( void)
{
	char	b[ 256];
	char	*p;

	p = json_buf;
	
	p[ 0] = 0;

	sprintf( b, "{");										p = strcat( p, b);

	sprintf( b, "\"ssid\":\"%s\","	, hard2.wifi_ssid);		p = strcat( p, b);
	sprintf( b, "\"key\":\"%s\","	, hard2.wifi_pawd);		p = strcat( p, b);
	sprintf( b, "\"ip\":\"%d.%d.%d.%d\","	, hard2.ip[ 0], hard2.ip[ 1], hard2.ip[ 2], hard2.ip[ 3]);	p = strcat( p, b);
	sprintf( b, "\"average\":%d,"	, para.cc_average);		p = strcat( p, b);
	sprintf( b, "\"wave1\":%d,"		, para.wave1);			p = strcat( p, b);
	sprintf( b, "\"wave2\":%d,"		, para.wave2);			p = strcat( p, b);
	sprintf( b, "\"fver\":\"firm version %s\""		, hard.firm);			p = strcat( p, b);
	
	sprintf( b, "}");										p = strcat( p, b);

	server.send(200, "application/json", json_buf);
}

void	cut_dasoku( char *p)
{
	int	i;
	int	len;

	len = 0;
	for( i = 0 ; i < 128; i++){
		if( p[ i] == 0){
			len = i;
			break;
		}
	}
	if( len > DASOKU_LEN){
		p[ len - DASOKU_LEN] = 0;
	}
}

void	handleConf1set( void)
{
	union	{
		char	b[ 32];
		char	*p[8];
	} d;
	char	b[ 256];
	char	*p;
	
	if( server.args()){
		sprintf( d.b, "%s", server.arg( 0));
		sprintf( b, "%s", d.p[ 0]);
		cut_dasoku( b);
		sprintf( hard2_chg.wifi_ssid, "%s", b);
		state.hard2_save = 1;
	}
	p = json_buf;
	p[ 0] = 0;
	sprintf( b, "{\"ssid\":\"%s\"}"	, hard2_chg.wifi_ssid);	p = strcat( p, b);
	server.send(200, "application/json", json_buf);
}

void	handleConf2set( void)
{
	union	{
		char	b[ 32];
		char	*p[8];
	} d;
	char	b[ 256];
	char	*p;
	
	if( server.args()){
		sprintf( d.b, "%s", server.arg( 0));
		sprintf( b, "%s", d.p[ 0]);
		cut_dasoku( b);
		sprintf( hard2_chg.wifi_pawd, "%s", b);
		state.hard2_save = 1;
	}
	p = json_buf;
	p[ 0] = 0;
	sprintf( b, "{\"pawd\":\"%s\"}"	, hard2_chg.wifi_pawd);	p = strcat( p, b);

	server.send(200, "application/json", json_buf);
}
void	handleConf3set( void)
{
	union	{
		char	b[ 32];
		char	*p[8];
	} d;
	char	b[ 256];
	char	*p;
	int		n;
	int		checknum;
	
	if( server.args()){
		sprintf( d.b, "%s", server.arg( 0));
		sprintf( b, "%s", d.p[ 0]);
		checknum = atoi( b);
		if( checknum > 0){
			hard2_chg.ip[ 0] = atoi( b);
			n = skip_until_code( b, '.');
			hard2_chg.ip[ 1] = atoi( &b[ n]);
			n += skip_until_code( &b[ n], '.');
			hard2_chg.ip[ 2] = atoi( &b[ n]);
			n += skip_until_code( &b[ n], '.');
			hard2_chg.ip[ 3] = atoi( &b[ n]);
			state.hard2_save = 1;
		}
	}
	p = json_buf;
	p[ 0] = 0;
	sprintf( p, "{\"ip\":\"%d.%d.%d.%d\"}"	, para_chg.ip[ 0], para_chg.ip[ 1], para_chg.ip[2], para_chg.ip[ 3]);

	server.send(200, "application/json", json_buf);
}

void	handleConf4set( void)
{
	char	b[ 256];
	char	*p;
	int		n;
	
	if( server.hasArg( "s")){
		sprintf( b, "%s", server.arg( "s"));
		para_chg.cc_average = atoi( b);
		state.para_save = 1;
	}
	p = json_buf;
	p[ 0] = 0;
	sprintf( b, "{\"average\":%d}"	, para_chg.cc_average);	p = strcat( p, b);

	server.send(200, "application/json", json_buf);
}

void	handleConf5set( void)
{
	char	b[ 256];
	char	*p;
	int		n;
	
	if( server.hasArg( "s")){
		sprintf( b, "%s", server.arg( "s"));
		para_chg.wave1 = atoi( b);
		state.para_save = 1;
	}
	p = json_buf;
	p[ 0] = 0;
	sprintf( b, "{\"wave1\":%d}"	, para_chg.wave1);	p = strcat( p, b);

	server.send(200, "application/json", json_buf);
}
void	handleConf6set( void)
{
	char	b[ 256];
	char	*p;
	int		n;
	
	if( server.hasArg( "s")){
		sprintf( b, "%s", server.arg( "s"));
		para_chg.wave2 = atoi( b);
		state.para_save = 1;
	}
	p = json_buf;
	p[ 0] = 0;
	sprintf( b, "{\"wave2\":%d}"	, para_chg.wave1);	p = strcat( p, b);

	server.send(200, "application/json", json_buf);
}

#define	MAX_LENGTH	280


char*	str_float_array( char *p, char *name, int s, int e, float *f)
{
	char	b[ 32];
	int		i;
	char	*x;


	x = p;
	sprintf( b, "\"%s\":[", name);
	p = strcat( p, b);
	for( i = s; i < e; i++){
		if( state.graph_mode == mode_raw){
			if( i < ( e-1)){
				sprintf( b, "%2.1f,", f[ i]);
			} else {
				sprintf( b, "%2.1f", f[ i]);
			}
		} else {
			if( i < ( e-1)){
				sprintf( b, "%4.3f,", f[ i]);
			} else {
				sprintf( b, "%4.3f", f[ i]);
			}
		}
		p = strcat( p, b);
	}
	p = strcat( p, "]");
	return p;
}

void	get_peak( float *p)
{
	int		i;
	float	peak;
	int		pw;
	
	peak 	= 0;
	pw		= 0;
	for( i = 0 ; i < MAX_WAVE; i++){
		if( p[ i] > peak){
			peak = p[ i];
			pw = i*2 + 340;
		}
	}
	state.peak = peak;
	state.pwave = pw;
}

float	get_ndiv( float *p)
{
	float	ret;
	int		w1, w2;

	w1 = (para.wave1 - 340)/2;
	w2 = (para.wave2 - 340)/2;
	if(( p[ w1] + p[ w2]) < 0.001){
		ret = 0;	
	} else {
		ret = ( p[ w1] - p[ w2])/( p[ w1] + p[ w2]);
	}
	return ret;
}

const	float	dummy[ MAX_WAVE] = {
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

void	get_pixel_data( char *p)
{
	char	b[ 256];
	int		i;
	int		kind;
	
	p[ 0] = 0;
	
	sprintf( b, "{\"n\":%d,"	, MAX_WAVE);					p = strcat( p, b);
	sprintf( b, "\"t\":%3.1f,"	, spectrum.temp);				p = strcat( p, b);
	sprintf( b, "\"v\":%3.1f,"	, spectrum.batv);				p = strcat( p, b);
	if( state.graph_mode == mode_raw){
		sprintf( b, "\"c\":%d,"		, state.ref_raw_no);		p = strcat( p, b);
	} else {
		sprintf( b, "\"c\":%d,"		, state.ref_reflect_no);	p = strcat( p, b);
	}
	sprintf( b, "\"e\":%d,"		, spectrum.exp);				p = strcat( p, b);
	sprintf( b, "\"f\":%d,"		, reference.exp);				p = strcat( p, b);
	sprintf( b, "\"a\":%d,"		, esp32_in.avg);				p = strcat( p, b);
	sprintf( b, "\"save\":%d,"	, state.data_save);				p = strcat( p, b);
	sprintf( b, "\"repeat\":%d,", state.graph_repeat);			p = strcat( p, b);
	if( state.graph_mode == mode_raw){
		if( state.graph_disp == disp_ref){
			get_peak( reference.data);
		} else {
			get_peak( spectrum.data);
		}
		sprintf( b, "\"mode\":\"raw\",");						p = strcat( p, b);
		sprintf( b, "\"msg\":\"peak Data %4.1f(%dnm)\",", state.peak, state.pwave);	p = strcat( p, b);
	} else {
		if( state.graph_disp == disp_ref){
			state.ndiv = get_ndiv( reference.data);
		} else {
			state.ndiv = get_ndiv( spectrum.data);
		}
		sprintf( b, "\"mode\":\"reflect\",");					p = strcat( p, b);
		sprintf( b, "\"msg\":\"NDIV=%4.3f( %d, %d )\",", state.ndiv, para.wave1, para.wave2);	p = strcat( p, b);
	}
	if(state.graph_disp == disp_data){
		sprintf( b, "\"disp\":\"data\",");						p = strcat( p, b);
	} else if( state.graph_disp == disp_comp){
		sprintf( b, "\"disp\":\"comp\",");						p = strcat( p, b);
	} else {
		sprintf( b, "\"disp\":\"ref\",");						p = strcat( p, b);
	}
	kind = state.graph_mode;
	if( kind == state.data_kind){
		p = str_float_array( p, "d"	, 0, MAX_WAVE, spectrum.data);
	} else {
		p = str_float_array( p, "d"	, 0, MAX_WAVE, (float*)dummy);
	}
	p = strcat( p, ",");
	sprintf( b, "\"tm\":\"%s\",", reference.tm);				p = strcat( p, b);
	p = str_float_array( p, "r"	, 0, MAX_WAVE, reference.data);
	sprintf( b, "}\n");											p = strcat( p, b);
}

void	send_data( void)
{
	get_pixel_data( json_buf);

	state.data_save	= 0;
	server.send(200, "application/json", json_buf);
}

void	init_web_buf( void)
{
	Serial.println( "web prepair start");
	get_html( html_buf);
	get_style( style_buf);
	get_javascript( js_buf);
	get_html_config( config_html_buf);
	get_js_config( config_js_buf);
	Serial.println( " web prepair end");
}

void	handleData( void)
{
	send_data();
}

void	handleUp1( void)
{
	char	b[ 64];
	int		i;

	if( esp32_out.exp < 500){
			esp32_out.exp = 500;			// 0.5ms
	} else if( esp32_out.exp < 1000){
			esp32_out.exp = 1000;			// 1ms
	} else if( esp32_out.exp < 2500){
			esp32_out.exp = 2500;			// 2.5ms
	} else if( esp32_out.exp < 5000){
			esp32_out.exp = 5000;			// 5ms
	} else if( esp32_out.exp < 10000){
			esp32_out.exp = 10000;			// 10ms
	} else if( esp32_out.exp < 25000){
			esp32_out.exp = 25000;			// 50ms
	} else if( esp32_out.exp < 50000){
			esp32_out.exp = 50000;			// 50ms
	} else if( esp32_out.exp < 100000){
			esp32_out.exp = 100000;			// 100ms
	}
	if( para_chg.cc_exposure != esp32_out.exp){
		para_chg.cc_exposure = esp32_out.exp;
		state.para_save = 1;
	}
	sprintf( b, "{\"msg\":\"set EXPOSURE UP\"}\n");
   server.send(200, "application/json", b);
}

void	handleUp2( void)
{
	char	b[ 64];
	int		i;

	if( esp32_out.exp < 500){
			esp32_out.exp += 50;			// 0.5ms
	} else if( esp32_out.exp < 1000){
			esp32_out.exp += 100;			// 1ms
	} else if( esp32_out.exp < 5000){
			esp32_out.exp += 500;			// 0.5ms
	} else if( esp32_out.exp < 10000){
			esp32_out.exp += 1000;			// 1ms
	} else if( esp32_out.exp < 50000){
			esp32_out.exp += 5000;			// 5ms
	} else if( esp32_out.exp < 100000){
			esp32_out.exp += 10000;			// 10ms
	}
	if( para_chg.cc_exposure != esp32_out.exp){
		para_chg.cc_exposure = esp32_out.exp;
		state.para_save = 1;
	}
	sprintf( b, "{\"msg\":\"set EXPOSURE UP\"}\n");
   server.send(200, "application/json", b);
}

void	handleDown1( void)
{
	char	b[ 64];
	int		i;

	if( esp32_out.exp > 50000){
			esp32_out.exp = 50000;			// 50ms
	} else if( esp32_out.exp > 25000){
			esp32_out.exp = 25000;			// 25ms
	} else if( esp32_out.exp > 10000){
			esp32_out.exp = 10000;			// 10ms
	} else if( esp32_out.exp > 5000){
			esp32_out.exp = 5000;			// 5ms
	} else if( esp32_out.exp > 2500){
			esp32_out.exp = 2500;			// 2.5ms
	} else if( esp32_out.exp > 1000){
			esp32_out.exp = 1000;			// 1ms
	} else if( esp32_out.exp > 500){
			esp32_out.exp = 500;			// 0.5ms
	} else {
			esp32_out.exp = 300;			// 0.3ms
	}
	if( para_chg.cc_exposure != esp32_out.exp){
		para_chg.cc_exposure = esp32_out.exp;
		state.para_save = 1;
	}
	sprintf( b, "{\"msg\":\"set exposure DOWN\"}\n");
	server.send(200, "application/json", b);
}

void	handleDown2( void)
{
	char	b[ 64];
	int		i;

	if( esp32_out.exp > 50000){
			esp32_out.exp -= 10000;			// 10ms
	} else if( esp32_out.exp > 10000){
			esp32_out.exp -= 5000;			// 1ms
	} else if( esp32_out.exp > 5000){
			esp32_out.exp -= 1000;			// 0.1ms
	} else if( esp32_out.exp > 1000){
			esp32_out.exp -= 500;			// 0.5ms
	} else if( esp32_out.exp > 500){
			esp32_out.exp -= 100;			// 0.1ms
	} else if( esp32_out.exp > 300){
			esp32_out.exp -= 50;			// 0.1ms
	}
	if( para_chg.cc_exposure != esp32_out.exp){
		para_chg.cc_exposure = esp32_out.exp;
		state.para_save = 1;
	}
	sprintf( b, "{\"msg\":\"set exposure DOWN\"}\n");
	server.send(200, "application/json", b);
}

void	handleRepeat( void)
{
	char	b[ 64];
	int		i;

	if( state.graph_repeat == 1){
		state.graph_repeat = 0;
		sprintf( b, "{\"msg\":\"set Monitor stop\"}\n");
	} else {
		state.graph_repeat = 1;
		sprintf( b, "{\"msg\":\"set Monitor start\"}\n");
	}
	server.send(200, "application/json", b);
}

void	handleMode( void)
{
	char	b[ 64];

	if( state.graph_mode == mode_raw){
		state.graph_mode = mode_reflect;
		sprintf( b, "{\"msg\":\"set Data Normalize\"}\n");
	} else {
		state.graph_mode = mode_raw;
		sprintf( b, "{\"msg\":\"set Data Raw mode\"}\n");
	}
	server.send(200, "application/json", b);
}

void	handleDisp( void)
{
	char	b[ 64];

	if( state.graph_disp == disp_data){
		state.graph_disp = disp_comp;
		if( state.graph_mode == mode_raw){
			state.kick_select = state.ref_raw_no;
		} else {
			state.kick_select = state.ref_reflect_no + 6;
		}
		sprintf( b, "{\"msg\":\"set Disp Compare\"}\n");
	} else if( state.graph_disp == disp_comp){
		state.graph_disp = disp_ref;
		sprintf( b, "{\"msg\":\"set Disp Reference\"}\n");
	} else {
		state.graph_disp = disp_data;
		sprintf( b, "{\"msg\":\"set Disp Data \"}\n");
	}
	server.send(200, "application/json", b);
}

void	handleStart( void)
{
	char	b[ 64];
	int		i;

	state.graph_mode = mode_raw;
	state.graph_repeat = 1;
	sprintf( b, "{\"msg\":\"start ColorCompassWiFi\"}\n");

   server.send(200, "application/json", b);
}

void	handleSave( void)
{
	char	b[ 64];
	int		i;

	state.data_save = 1;
	sprintf( b, "{\"msg\":\"set Data Save CSV File\"}\n");

   server.send(200, "application/json", b);
}

void	white_set( void)
{
	int	i;
	
	for( i = 0 ; i < MAX_PIXEL; i++){
		para_chg.pixel_white[ i] = esp32_in.pixel[ i];
	}
	state.para_save = 1;
}

void	white_clear( void)
{
	int	i;
	
	for( i = 0 ; i < MAX_PIXEL; i++){
		para_chg.pixel_white[ i] = 65535;
	}
	state.para_save = 1;
}


void	handleWhiteSet( void)
{
	char	b[ 64];
	state.kick_white = 1;
	sprintf( b, "{\"msg\":\"set Now Data to White\"}\n");
	server.send(200, "application/json", b);
}

void	handleWhiteClear( void)
{
	char	b[ 64];

	state.kick_white = 2;
	sprintf( b, "{\"msg\":\"reset White data(=65535)\"}\n");
	server.send(200, "application/json", b);
}

void	handleDarkSet( void)
{
	char	b[ 64];
	int		i;

	state.kick_dark = 1;
	sprintf( b, "{\"msg\":\"set Now Data to Dark\"}\n");
	server.send(200, "application/json", b);
}

void	handleDarkClear( void)
{
	char	b[ 64];
	
	state.kick_dark = 2;
	sprintf( b, "{\"msg\":\"reset Dark data\"}\n");
	server.send(200, "application/json", b);
}

void	set_ref( int no, int m, int d)
{
	union	{
		char	b[ 32];
		char	*p[8];
	} r;
	char	b[ 256];
	char	*p;
	

	if( d == disp_data){						// regist
		if( server.args()){
			sprintf( r.b, "%s", server.arg( 0));
			sprintf( spectrum.tm, "%s", r.p[ 0]);
		}
		if( m == mode_raw){
			state.kick_regist = no;
			sprintf( b, "{\"msg\":\"Ref #%d registered\"}\n", no);
		} else if( m == mode_reflect){
			state.kick_regist = no + 6;
			sprintf( b, "{\"msg\":\"Ref #%d registered\"}\n", no);
		}
	} else {									// select
		if( m == mode_raw){
			state.kick_select = no;
			state.ref_raw_no = no;
			sprintf( b, "{\"msg\":\"Ref #%d select\"}\n", no);
		} else {
			state.kick_select = no+6;
			state.ref_reflect_no = no;
			sprintf( b, "{\"msg\":\"Ref #%d select\"}\n", no);
		}
	}
	server.send(200, "application/json", b);
}

void	handleRef1( void)
{

	set_ref( 1, state.graph_mode, state.graph_disp);
}

void	handleRef2( void)
{
	set_ref( 2, state.graph_mode, state.graph_disp);
}

void	handleRef3( void)
{
	set_ref( 3, state.graph_mode, state.graph_disp);
}

void	handleRef4( void)
{
	set_ref( 4, state.graph_mode, state.graph_disp);
}

void	handleRef5( void)
{
	set_ref( 5, state.graph_mode, state.graph_disp);
}

void	handleRef6( void)
{
	set_ref( 6, state.graph_mode, state.graph_disp);
}

void	use_dark( void)
{
	int		i;

	for( i = 0 ; i < MAX_PIXEL; i++){
		dark[ i] = hard.pixel_offset[ i];
	}
}
void	dark_set( void)
{
	int	i;
	
	for( i = 0 ; i < MAX_PIXEL; i++){
		dark[ i]	= esp32_in.pixel[ i];
	}
}

void	dark_clear( void)
{
	int	i;
	
	for( i = 0 ; i < MAX_PIXEL; i++){
		dark[ i] = hard.pixel_offset[ i];
	}
	state.para_save = 1;
}



void handleNotFound()
{
	String message = "File Not Found\n\n";
	message += "URI: ";
	message += server.uri();
	message += "\nMethod: ";
	message += (server.method() == HTTP_GET) ? "GET" : "POST";
	message += "\nArguments: ";
	message += server.args();
	message += "\n";
	for (uint8_t i = 0; i < server.args(); i++) {
		message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
	}
	server.send(404, "text/plain", message);
}

void	set_server( void)
{
		server.on( "/", handleRoot);
		server.on( "/style.css", handleStyle);
		server.on( "/index.js", handleJavaScript);
		server.on( "/data/data.json", handleData);
		server.on( "/data/up1.json", handleUp1);
		server.on( "/data/up2.json", handleUp2);
		server.on( "/data/down1.json", handleDown1);
		server.on( "/data/down2.json", handleDown2);
		server.on( "/data/start.json", handleStart);
		server.on( "/data/save.json", handleSave);
		server.on( "/data/repeat.json", handleRepeat);
		server.on( "/data/mode.json", handleMode);
		server.on( "/data/disp.json", handleDisp);
		server.on( "/data/wset.json", handleWhiteSet);
		server.on( "/data/wclear.json", handleWhiteClear);
		server.on( "/data/dset.json", handleDarkSet);
		server.on( "/data/dclear.json", handleDarkClear);
		server.on( "/data/ref1.json", handleRef1);
		server.on( "/data/ref2.json", handleRef2);
		server.on( "/data/ref3.json", handleRef3);
		server.on( "/data/ref4.json", handleRef4);
		server.on( "/data/ref5.json", handleRef5);
		server.on( "/data/ref6.json", handleRef6);
		server.on( "/config.html", handleHtmlConfig);
		server.on( "/config.js", handleJsConfig);
		server.on( "/data/config.json", handleConfigJson);
		server.on( "/data/confset1.json", handleConf1set);
		server.on( "/data/confset2.json", handleConf2set);
		server.on( "/data/confset3.json", handleConf3set);
		server.on( "/data/confset4.json", handleConf4set);
		server.on( "/data/confset5.json", handleConf5set);
		server.on( "/data/confset6.json", handleConf6set);
		server.onNotFound(handleNotFound);
}
char	wifi_connecting( void)
{
	if( WiFi.waitForConnectResult() == WL_CONNECTED) {
		MDNS.begin( host);
		if( MDNS.begin( "esp32")) {
			Serial.println( "mDNS responder started");
		}
		set_server();
		server.begin();
		Serial.println( "HTTP server started");
		return true;
	} else {
		return false;
	}
}


char	softap_start( void)
{
		set_server();
		server.begin();
		return 1;
}

unsigned int get_checksum( unsigned int *p, int size)
{
	int	 i;
	unsigned int	sum;
	
	sum = 0;
	for( i = 0 ; i < size; i++){
		sum += *p++;
	}
	return sum;
}


int	comp_str( char *s, char *d)
{
	int	i;
	
	for( i = 0 ; i < 16; i++){
		if( *s++ != *d++) return 0;
		if(( *s == 0) || ( *d == 0)){
			break;
		}
	}
	return 1;
}

void	save_hard( void)
{
	hard.size = sizeof( hard);
	hard.sum = get_checksum( (unsigned int*)&hard, sizeof( hard)/4-2);
	eep_write( HARD_ADR, (char*)&hard, sizeof( hard));
}

int	save_hard_change( int adr)
{
	if( adr == 0){;
		hard_chg.size = sizeof( hard_chg);
		hard_chg.sum = get_checksum( (unsigned int*)&hard_chg, sizeof( hard_chg)/4-2);
		adr = eep_write_change( HARD_ADR, adr, (char*)&hard, (char*)&hard_chg, EP_WPAGE);
	} else {
		adr = eep_write_change( HARD_ADR, adr, (char*)&hard, (char*)&hard_chg, EP_WPAGE);
	}
	return adr;
}

void	save_hard2( void)
{
	hard2.size = sizeof( hard2);
	hard2.sum = get_checksum( (unsigned int*)&hard2, sizeof( hard2)/4-2);
	eep_write( HARD2_ADR, (char*)&hard2, sizeof( hard2));
}

int	save_hard2_change( int adr)
{
	if( adr == 0){;
		hard2_chg.size = sizeof( hard2_chg);
		hard2_chg.sum = get_checksum( (unsigned int*)&hard2_chg, sizeof( hard2_chg)/4-2);
		adr = eep_write_change( HARD2_ADR, adr, (char*)&hard2, (char*)&hard2_chg, EP_WPAGE);
	} else {
		adr = eep_write_change( HARD2_ADR, adr, (char*)&hard2, (char*)&hard2_chg, EP_WPAGE);
	}
	return adr;
}


void	save_para( void)
{
	para.size = sizeof( para);
	para.sum = get_checksum( (unsigned int*)&para, sizeof( para)/4-2);
	eep_write( PARA_ADR, (char*)&para, sizeof( para));
}

int	save_para_change( int adr)
{
	if( adr == 0){;
		para_chg.size = sizeof( para_chg);
		para_chg.sum = get_checksum( (unsigned int*)&para_chg, sizeof( para_chg)/4-2);
		adr = eep_write_change( PARA_ADR, adr, (char*)&para, (char*)&para_chg, EP_WPAGE);
	} else {
		adr = eep_write_change( PARA_ADR, adr, (char*)&para, (char*)&para_chg, EP_WPAGE);
	}
	return adr;
}


void	load_hard( void)
{
	unsigned int	sum;
	int				i;
	char			b[ 32];
	
	eep_read( HARD_ADR, (char*)&hard, sizeof( hard));
	sum = get_checksum( (unsigned int*)&hard, sizeof( hard)/4-2);
	if(( sum != hard.sum) || ( hard.size  != sizeof( hard))){
		sprintf( b, "checksum = %08X %08X", sum, hard.sum);
		Serial.println( b);
		hard.size		= sizeof( hard);
		sprintf( hard.firm, 	firm_ver);				// firm version
		sprintf( hard.cc_name,		"colorcompassWiFi");
		sprintf( hard.cc_sensor,	"C12880MA");
		sprintf( hard.cc_serial,	"00X00000");
		hard.cc_comp[ 0] =  3.150949296E02;				// "14J00004"
		hard.cc_comp[ 1] =  2.700213191E00;
		hard.cc_comp[ 2] = -1.053135078E-3;
		hard.cc_comp[ 3] = -9.414243769E-6;
		hard.cc_comp[ 4] =  1.836400722E-8;
		hard.cc_comp[ 5] = -1.019880984E-11;
		for( i = 0 ; i < MAX_PIXEL; i++){
			hard.pixel_offset[ i] = 0;
			hard.pixel_gain[ i] = 1.0;
		}
		save_hard();
	} else if( !comp_str( (char*)firm_ver, hard.firm)){
		sprintf( hard.firm, 	firm_ver);			// firm version
		save_hard();
	}
	copy_len( (char*)&hard, (char*)&hard_chg, sizeof( hard));
}

void	load_hard2( void)
{
	unsigned int	sum;
	int				i;
	char			b[ 32];
	
	eep_read( HARD2_ADR, (char*)&hard2, sizeof( hard2));
	sum = get_checksum( (unsigned int*)&hard2, sizeof( hard2)/4-2);
	if(( sum != hard2.sum) || ( hard2.size  != sizeof( hard2))){
		for( i = 0 ; i < 32; i++){
			if( i < 16){
				hard2.wifi_ssid[ i] = para.wifi_ssid[ i];
				hard2.wifi_pawd[ i] = para.wifi_pawd[ i];
			} else {
				hard2.wifi_ssid[ i] = 0;
				hard2.wifi_pawd[ i] = 0;
			}
		}
		hard2.ip[ 0] = para.ip[ 0];
		hard2.ip[ 1] = para.ip[ 1];
		hard2.ip[ 2] = para.ip[ 2];
		hard2.ip[ 3] = para.ip[ 3];
		save_hard2();
	}
	copy_len( (char*)&hard2, (char*)&hard2_chg, sizeof( hard2));
}

void	load_para( void)
{
	unsigned int	sum;
	int				i;
	char			b[ 32];
	
	eep_read( PARA_ADR, (char*)&para, sizeof( para));
	sum = get_checksum( (unsigned int*)&para, sizeof( para)/4-2);
	if(( sum != para.sum) || ( para.size  != sizeof( para))){
		sprintf( b, "checksum = %08X %08X", sum, para.sum);
		Serial.println( b);
		para.size		= sizeof( para);
		sprintf( para.wifi_ssid, 	"------------");
		sprintf( para.wifi_pawd, 	"-------------");
		para.ip[ 0] = 192;
		para.ip[ 1] = 168;
		para.ip[ 2] = 0;
		para.ip[ 3] = 100;
		para.cc_exposure	= 1000;
		para.cc_average	= 10;
		para.wave1		= 680;
		para.wave2		= 850;
		save_para();
	}
	copy_len( (char*)&para, (char*)&para_chg, sizeof( para));
}

#define JST     3600* 9

int	printLocalTime()
{
	struct tm timeinfo;
	
	if( getLocalTime(&timeinfo)){
		Serial.println( &timeinfo, "%A, %B %d %Y %H:%M:%S");
		return 1;
	} else {
	    Serial.println( "Failed to obtain time");
	    return 0;
	}
}

void	calc_wave( double *p)
{
	int		i;

	for( i = 1 ; i <= MAX_PIXEL ; i++){
		pixel_wave[ i-1] = p[ 0] + p[ 1] * i + p[ 2] * i * i + p[ 3] * i * i * i + p[ 4] * i * i * i * i + p[ 5] * i * i * i * i * i;
	}
}

void	subtract_dark( float *p, unsigned short *pixel, unsigned short *dark)
{
	int	i;

	for( i = 0 ; i < MAX_PIXEL; i++){
		p[ i] = pixel[ i] - dark[ i];
		if( p[i]< 0) p[ i] = 0;
	}
}

void	calc_gain( float *p, float *gain)
{
	int	i;

	for( i = 0 ; i < MAX_PIXEL; i++){
		p[ i] = p[ i] * gain[ i];
	}
}

void	calc_reflection( float *p,unsigned short *white, unsigned short *dark)
{
	int		i;
	float	d;
	
	for( i = 0 ; i < MAX_PIXEL; i++){
		d = white[ i] - dark[ i];
		if( d < 1.0) d = 1.0;
		p[ i] = p[ i] / d;
	}
}

void	calc_wave_data( float *wave, float *s, float *dist)
{
	int		i, j, n;
	float	w;
	float	calc;

	n = 0;
	j = 0;
	for( i = START_WAVE; i <= END_WAVE; i += 2){
		w = (float)i;
		for( ; j < MAX_PIXEL; j++){
			if(( w > wave[ j]) && ( w <= wave[ j+1])){				// i = 350, 351, 352, ...wave[ ] = 340.4, 343.7, ...
				break;
			}
		}
		if( j < MAX_PIXEL){
			dist[ n++] = ((float)s[ j+1] - (float)s[ j]) * ( w - wave[ j])/( wave[ j+1] - wave[ j]) + (float)s[ j];
		} else {
			dist[ n++] = 0;
		}
	}
}

void	disp_para( void)
{
	char	b[ 128];
	int		i;
	String	str;
	
	sprintf( b, "firm v:%s", hard.firm);				Serial.println( b);
	sprintf( b, "para  :%d", sizeof( para));			Serial.println( b);
	sprintf( b, "ssid  :%s", hard2.wifi_ssid);			Serial.println( b);
	sprintf( b, "pswd  :%s", hard2.wifi_pawd);			Serial.println( b);
	sprintf( b, "ip    :%d.%d.%d.%d", hard2.ip[ 0], hard2.ip[ 1], hard2.ip[ 2], hard2.ip[ 3]);	Serial.println( b);
	sprintf( b, "name  :%s", hard.cc_name);				Serial.println( b);
	sprintf( b, "exp   :%d", para.cc_exposure);			Serial.println( b);
	sprintf( b, "avg   :%d", para.cc_average);			Serial.println( b);
	sprintf( b, "sensor:%s", hard.cc_sensor);			Serial.println( b);
	sprintf( b, "serial:%s", hard.cc_serial);			Serial.println( b);
	sprintf( b, "comp#0:%10.8e", hard.cc_comp[ 0]);		Serial.println( b);
	sprintf( b, "comp#1:%10.8e", hard.cc_comp[ 1]);		Serial.println( b);
	sprintf( b, "comp#2:%10.8e", hard.cc_comp[ 2]);		Serial.println( b);
	sprintf( b, "comp#3:%10.8e", hard.cc_comp[ 3]);		Serial.println( b);
	sprintf( b, "comp#4:%10.8e", hard.cc_comp[ 4]);		Serial.println( b);
	sprintf( b, "comp#5:%10.8e", hard.cc_comp[ 5]);		Serial.println( b);
	sprintf( b, "wave1 :%d", para.wave1);				Serial.println( b);
	sprintf( b, "wave2 :%d", para.wave2);				Serial.println( b);

	sprintf( b, "Dark%2d:", 0);
	str = b;
	for( i = 0 ; i < MAX_PIXEL; i++){
		sprintf( b, "%d,", hard.pixel_offset[ i]);
		str += b;
		if(( i % 8) == 7){
			Serial.println( str);
			sprintf( b, "Dark%2d:", (i+1)/8);
			str = b;
		}
	}
/*	sprintf( b, "Dark%2d:", MAX_PIXEL/8);
	str = b;
	for( i = 0 ; i < MAX_PIXEL; i++){
		sprintf( b, "%d,", para.pixel_offset[ i+MAX_PIXEL]);
		str += b;
		if(( i % 8) == 7){
			Serial.println( str);
			sprintf( b, "Dark%2d:", (i+1+MAX_PIXEL)/8);
			str = b;
		}
	}
*/
	sprintf( b, "Gain%2d:", 0);
	str = b;
	for( i = 0 ; i < MAX_PIXEL; i++){
		sprintf( b, "%5.3f,", hard.pixel_gain[ i]);
		str += b;
		if(( i % 8) == 7){
			Serial.println( str);
			sprintf( b, "Gain%2d:", (i+1)/8);
			str = b;
		}
	}

}

void	get_command( char *p)
{
	int		n;
	char	r;
	
	n = 0;
	while( 1){
		if( Serial.available()){
			r = Serial.read();
			p[ n++] = r;
			if( r == '\n'){
				p[ n] = 0;
				break;
			}
		}
	}
}

int	skip_until_code( char *p, char code)
{
	int		i;

	for( i = 0 ; ; i++){
		if( p[ i] == code){
			i++;
			return i;
		} else if( p[ i] == 0){
			break;
		}
	}
	return 0;
}

void	reject_ret( char *p)
{
	int		i;
	
	for( i = 0 ; ; i++){
		if( p[ i] == '\n'){
			p[ i] = 0;
			break;
		} else if( p[ i] == '\r'){
			p[ i] = 0;
			break;
		} else if( p[ i] == 0){
			break;
		}
	}
}

void	set_ssid( char *p)
{
	reject_ret( p);
	sprintf( hard2_chg.wifi_ssid, "%s", p);
}

void	set_password( char *p)
{
	reject_ret( p);
	sprintf( hard2_chg.wifi_pawd, "%s", p);
}

void	set_exposure( char *p)
{
	para_chg.cc_exposure = atoi( p);
	esp32_out.exp = para_chg.cc_exposure;
}

void	set_average( char *p)
{
	para_chg.cc_average = atoi( p);
}

void	set_serial( char *p)
{
	reject_ret( p);
	sprintf( hard_chg.cc_serial, "%s", p);
}

void	set_comp( char *p)
{
	int	no;
	int	n;
	
	no = atoi( p);
	n = skip_until_code( p, ',');
	hard_chg.cc_comp[ no] = atof( &p[ n]);
}

void	set_wave( char *p)
{
	int	no;
	int	n;
	
	no = atoi( p);
	n = skip_until_code( p, ',');
	if( n == 0){
		para_chg.wave1 = atof( &p[ n]);
	} else {
		para_chg.wave2 = atof( &p[ n]);
	}
}

void	set_dark( char *p)				// pixel data -> dark
{
	int	no;
	int	i;
	
	for( i = 0 ; i < MAX_PIXEL; i++){
		hard_chg.pixel_offset[ i]	= esp32_in.pixel[ i];
		dark[ i]				= hard_chg.pixel_offset[ i];
	}
}

void	set_gain( char *p)
{
	int	no;
	int	n;
	
	no = atoi( p);
	n = skip_until_code( p, ',');
	hard_chg.pixel_gain[ no] = atof( &p[ n]);
	
}

void	set_reference( char *p)
{
	int	no;
	int	n;
	
	no = atoi( p);
	n = skip_until_code( p, ',');
	reference.data[ no] = atof( &p[ n]);
}

void	save_reference( char *p)
{
	int	no;
	int	n;
	
	no = atoi( p);
	registReference( no);
}

void	set_ip( char *p)
{
	int	no;
	int	n;
	char	b[ 32];
	
	hard2_chg.ip[ 0] = atoi( p);
	n = skip_until_code( p, '.');
	hard2_chg.ip[ 1] = atoi( &p[n]);
	n += skip_until_code( &p[ n], '.');
	hard2_chg.ip[ 2] = atoi( &p[n]);
	n += skip_until_code( &p[ n], '.');
	hard2_chg.ip[ 3] = atoi( &p[n]);	
}

void	get_graph_mode( char *p)
{
	char	b[ 128];

	sprintf( b, "GraphMode,%d", state.graph_mode);
	Serial.println( b);
}

void	get_pixel( char *p)
{
	char	b[ 4096];
	int		i;
	char	*s;
	char	t[ 12];
	s = b;
	s[0] = 0;
	for( i = 0 ; i < MAX_PIXEL; i++){
		sprintf( t, "%4d,", esp32_in.pixel[ i]);
		s = strcat( s, t);
	}
	Serial.println( b);
}

void	get_spectrum( char *p)
{
	char	b[ 4096];
	int		i;
	char	*s;
	char	t[ 12];
	s = b;
	s[0] = 0;
	for( i = 0 ; i < MAX_WAVE; i++){
		if( state.graph_mode == mode_raw){
			sprintf( t, "%4.0f,", spectrum.data[ i]);
		} else {
			sprintf( t, "%4.3f,", spectrum.data[ i]);
		}
		s = strcat( s, t);
	}
	Serial.println( b);
	
}

void	loadRef( int no);

void	get_reference( char *p)
{
	char	b[ 4096];
	int		i;
	char	*s;
	int		no;
	char	t[ 12];

	no = atoi( p);
	loadRef( no+1);
	s = b;
	s[0] = 0;
	for( i = 0 ; i < MAX_WAVE; i++){
		if( no < 6){
			sprintf( t, "%4.0f,", reference.data[ i]);
		} else {
			sprintf( t, "%4.3f,", reference.data[ i]);
		}
		s = strcat( s, t);
	}
	Serial.println( b);
}

void	do_setting( void)
{
	char	in[32];
	char	cmd;
	int		n;
	
	if( Serial.available()){
		get_command( in);
		cmd = in[ 0];
		n = skip_until_code( in, ',');
		switch( cmd){
		case '\n':	disp_para();				break;			// get all data
		case 'i':	set_ip( &in[ n]);			break;			// set ip address
		case 's':	set_ssid( &in[ n]);			break;			// set ssid
		case 'p':	set_password( &in[ n]);		break;			// set password
		case 'a':	set_average( &in[ n]);		break;			// set average
		case 'e':	set_exposure( &in[ n]);		break;			// set exposure
		case 'r':	set_serial( &in[ n]);		break;			// set sensor serial no.
		case 'c':	set_comp( &in[ n]);			break;			// set sensor comp data
		case 'd':	set_dark( &in[ n]);			break;			// set sensor dark data
		case 'g':	set_gain( &in[ n]);			break;			// set sensor gain data
		case 'f':	set_reference( &in[ n]);	break;			// set reference data
		case 'h':	save_reference( &in[ n]);	break;			// save reference data
		case 'v':	set_wave( &in[ n]);			break;			// set sensor wave data
		case 'x':	state.para_save = 1;		break;			// save parameter
		case 'y':	state.hard_save = 1;		break;			// save hard
		case 'u':	state.hard2_save = 1;		break;			// save hard2
		case 'z':	while(1);					break;			// restart ( WDT )
		case 'D':	get_spectrum( in);			break;			// get spectrum data
		case 'P':	get_pixel( in);				break;			// get pixel data
		case 'F':	get_reference( &in[n]);		break;			// get reference data
		case 'G':	get_graph_mode( in);		break;			// get graph mode
		}
		switch( cmd){
		case 'm':		// set mode
		case 'w':		// set wifi
		case 's':		// set ssid
		case 'p':		// set password
		case 'a':		// set average
		case 'e':		// set exposure
		case 'r':		// set sensor serial no.
		case 'c':		// set sensor comp data
		case 'd':		// set sensor dark data
		case 'v':		// set wave num
		case 'g':		// set sensor gain data
		case 'f':		// set reference data
		case 'h':		// save reference data
		case 'i':		// set ip address
		case 'x':		// save parameter
		case 'y':		// save hard
		case 'u':		// save hard2
					Serial.println( "OK");
					break;
		}
	}
}


void	registData( int no)
{
	spectrum.sum = get_checksum( (unsigned int*)&spectrum, sizeof( spectrum)/4-2);
	eep_write( REF_ADR + (no-1) * REF_STEP, (char*)&spectrum, sizeof( spectrum));
}

void	registReference( int no)
{
	reference.sum = get_checksum( (unsigned int*)&reference, sizeof( reference)/4-2);
	eep_write( REF_ADR + (no-1) * REF_STEP, (char*)&reference, sizeof( reference));
}

void	loadRef( int no)
{
	int		i;

	eep_read( REF_ADR + (no-1) * REF_STEP, (char*)&reference, sizeof( reference));
	if( reference.sum != get_checksum( (unsigned int*)&reference, sizeof( reference)/4-2)){
		for( i = 0 ; i < MAX_WAVE; i++){
			reference.data[ i] = 0.0;
		}
		reference.exp	= 500;
		reference.temp	= 30.0;
		reference.batv	= 8.0;
		sprintf( reference.tm, "");
	}
}


void setup(void) {
	char	b[ 64];
	int		i;
		
	state.wifi_ok 		= 0;
	state.graph_repeat	= 1;
	state.data_save		= 0;
	state.kick_data		= 0;
	state.para_save		= 0;
	state.para_adr		= 0;
	state.graph_mode	= mode_raw;
	state.graph_disp	= disp_data;
	state.ref_raw_no	= 1;
	state.ref_reflect_no	= 1;
	state.kick_select	= 1;
	state.hard_save		= 0;
	state.hard_adr		= 0;
	state.hard2_save	= 0;
	state.hard2_adr		= 0;

	pinMode(IO23,	INPUT_PULLUP);
	
	Serial.begin(115200);
	Serial.println();
	spi_init();

	eep_init();
	if( eep_ready( EEP_ADRS0) && eep_ready( EEP_ADRS1)){
		Serial.println( "eep ready");
		state.eeprom_ok = 1;
	} else {
		Serial.println( "eep not ready");
		state.eeprom_ok = 0;
	}
	load_hard();
	load_para();
	load_hard2();
	loadRef( state.kick_select);
	adcAttachPin(AN6);
	analogSetAttenuation( ADC_2_5db);
	//analogReadMilliVolts( an);
	esp32_in.exp = 123;
	esp32_out.exp = para.cc_exposure;
	esp32_out.avg = para.cc_average;
	if( get_sw()){
		state.mode = mode_AP;
	} else{
		state.mode	= mode_STA;
	}
	if( state.mode == mode_STA){			// STA
		WiFi.mode(WIFI_STA);
		WiFi.begin( hard2.wifi_ssid, hard2.wifi_pawd);
		state.wifi_ok = wifi_connecting();
	} else {															// AP
		sprintf( b, "CC%s", hard.cc_serial);
		WiFi.mode(WIFI_AP);
	    WiFi.softAP( b, b);
	    state.wifi_ok = softap_start();
	}
	if( state.wifi_ok){
		if( state.mode == mode_STA){									// STA
//			WiFi.config( hard2.ip, WiFi.gatewayIP(), WiFi.subnetMask());
			WiFi.config( hard2.ip, WiFi.gatewayIP(), subnetmask);
			Serial.printf("Connected, IP address: ");
			Serial.println(WiFi.localIP());
		} else {
//			WiFi.softAPConfig( hard2.ip, WiFi.gatewayIP(),WiFi.subnetMask());
			WiFi.softAPConfig( ap_ip, WiFi.gatewayIP(),subnetmask);
			Serial.printf("Connected, IP address: ");
			Serial.println( WiFi.softAPIP());
		}
	}
	init_web_buf();
	for( i = 0 ; i < MAX_WAVE; i++){
		spectrum.data[ i] = i * 10.0;
		reference.data[ i] = i*100;
	}
	calc_wave( hard.cc_comp);
	use_dark();
	spi_start();
}

void loop(void) {
	int a6, a7;
	char b[ 32];
	int	 i;

	delay(1);
	if( state.wifi_ok){
		server.handleClient();
  	}
  	do_setting();
  	batv = analogReadMilliVolts( AN6)*11/1000.0;
  	if( state.para_save){
   		if( state.eeprom_ok){
	 		state.para_adr = save_para_change( state.para_adr);
	 		if( state.para_adr > sizeof( para)){
		 		state.para_save = 0;
				state.para_adr = 0;
				para = para_chg;
	 		}
  	 	} else {
	 		state.para_save = 0;
  	 	}
  	} else if( state.hard_save){
   		if( state.eeprom_ok){
	 		state.hard_adr = save_hard_change( state.hard_adr);
	 		if( state.hard_adr > sizeof( hard)){
		 		state.hard_save = 0;
				state.hard_adr = 0;
				hard = hard_chg;
	 		}
  	 	} else {
	 		state.hard_save = 0;
  	 	}
  	} else if( state.hard2_save){
   		if( state.eeprom_ok){
	 		state.hard2_adr = save_hard2_change( state.hard2_adr);
	 		if( state.hard2_adr > sizeof( hard2)){
		 		state.hard2_save = 0;
				state.hard2_adr = 0;
				hard2 = hard2_chg;
	 		}
  	 	} else {
	 		state.hard2_save = 0;
  	 	}
  	} else {
		state.para_adr = PARA_ADR;
  		if( state.kick_white == 1){				// set white
	  		state.kick_white = 0;
	  		white_set();
	  	} else if( state.kick_white == 2){		// clear white
	  		state.kick_white = 0;
	  		white_clear();
	  	} else if( state.kick_dark == 1){		// set dark
	  		state.kick_dark = 0;
	  		dark_set();
	  	} else if( state.kick_dark == 2){		// clear dark
	  		state.kick_dark = 0;
	  		dark_clear();
		} else if( state.kick_data == 1){		// subtract dark from pixel data
			esp32_out.index = esp32_in.index;
			spectrum.exp	= esp32_in.exp;
			spectrum.temp	= esp32_in.temperature;
			spectrum.batv	= batv;
	  		state.kick_data++;
	  		subtract_dark( pixel_data, esp32_in.pixel, dark);
	  	} else if( state.kick_data == 2){		// calc spectrum data from pixel data
	  		state.kick_data++;
	  		if( state.graph_mode == mode_raw){
				calc_gain( pixel_data, hard.pixel_gain);
	  		} else {
	  			calc_reflection( pixel_data, para.pixel_white, dark);
	  		}
	  	} else if( state.kick_data == 3){		// calc spectrum data from pixel data
			state.kick_data = 0;
			calc_wave_data( pixel_wave, pixel_data, spectrum.data);
			state.data_kind = state.graph_mode;
	  	} else if( state.kick_regist){			// regist reference data
	  		registData( state.kick_regist);
	  		state.kick_regist = 0;
	  	} else if( state.kick_select){			// select reference data
	  		loadRef( state.kick_select);
	  		state.kick_select = 0;
	  	}
	  	if( esp32_out.avg != para.cc_average){
	  		esp32_out.avg = para.cc_average;
	  	}
  	}
}
